# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from unittest import mock
from nose.tools import assert_raises

from superdesk.tests import TestCase
from superdesk.media.crop import CropService
from superdesk.errors import SuperdeskApiError
from superdesk.media.media_operations import crop_image
from superdesk.media.renditions import _resize_image, get_renditions_spec, can_generate_custom_crop_from_original
from apps.prepopulate.app_populate import populate_table_json

from ..media import get_picture_fixture


class CropTestCase(TestCase):

    crop_sizes = {
        "_id": "crop_sizes",
        "display_name": "Image Crop Sizes",
        "type": "manageable",
        "items": [
            {"is_active": True, "name": "4-3", "width": 800, "height": 600},
            {"is_active": True, "name": "16-9", "width": 1280, "height": 720}
        ]
    }

    def setUp(self):
        self.service = CropService()
        populate_table_json('vocabularies', [self.crop_sizes])

    def test_validate_aspect_ratio_fails(self):
        doc = {'CropLeft': 0, 'CropRight': 80, 'CropTop': 0, 'CropBottom': 60}
        crop = {'height': 700, 'width': 70}
        with assert_raises(SuperdeskApiError):
            self.service._validate_aspect_ratio(crop, doc)

    def test_validate_aspect_ratio_fails_with_cropsize_less(self):
        doc = {'CropLeft': 0, 'CropRight': 80, 'CropTop': 0, 'CropBottom': 60}
        crop = {'height': 600, 'width': 800}
        with assert_raises(SuperdeskApiError):
            self.service._validate_aspect_ratio(crop, doc)

    def test_validate_aspect_ratio_succeeds(self):
        doc = {'CropLeft': 0, 'CropRight': 800, 'CropTop': 0, 'CropBottom': 600}
        crop = {'height': 600, 'width': 800}
        self.assertIsNone(self.service._validate_aspect_ratio(crop, doc))

    def test_validate_aspect_ratio_succeeds_2(self):
        doc = {'CropLeft': 0, 'CropRight': 1600, 'CropTop': 0, 'CropBottom': 1200}
        crop = {'height': 600, 'width': 800}
        self.assertIsNone(self.service._validate_aspect_ratio(crop, doc))

    def test_get_crop_by_name(self):
        self.assertIsNotNone(self.service.get_crop_by_name('16-9'))
        self.assertIsNotNone(self.service.get_crop_by_name('4-3'))
        self.assertIsNone(self.service.get_crop_by_name('d'))

    def test_validate_crop_raises_error_if_item_is_not_picture(self):
        original = {"type": "text"}
        doc = {'renditions': {'4-3': {'CropLeft': 0, 'CropRight': 800, 'CropTop': 0, 'CropBottom': 600}}}
        with self.assertRaises(SuperdeskApiError) as context:
            self.service.validate_crop(original, doc, "4-3")

        ex = context.exception
        self.assertEqual(ex.message, 'Only images can be cropped!')
        self.assertEqual(ex.status_code, 400)

    def test_validate_crop_raises_error_if_renditions_are_missing(self):
        original = {"type": "picture"}
        doc = {'renditions': {'4-3': {'CropLeft': 0, 'CropRight': 800, 'CropTop': 0, 'CropBottom': 600}}}
        with self.assertRaises(SuperdeskApiError) as context:
            self.service.validate_crop(original, doc, "4-3")

        ex = context.exception
        self.assertEqual(ex.message, 'Missing renditions!')
        self.assertEqual(ex.status_code, 400)

    def test_validate_crop_raises_error_if_original_rendition_is_missing(self):
        original = {"type": "picture",
                    "renditions": {"4-3": {'CropLeft': 0, 'CropRight': 800, 'CropTop': 0, 'CropBottom': 600}}}
        doc = {'renditions': {'4-3': {'CropLeft': 0, 'CropRight': 800, 'CropTop': 0, 'CropBottom': 600}}}
        with self.assertRaises(SuperdeskApiError) as context:
            self.service.validate_crop(original, doc, "4-3")

        ex = context.exception
        self.assertEqual(ex.message, 'Missing original rendition!')
        self.assertEqual(ex.status_code, 400)

    def test_validate_crop_raises_error_if_crop_name_is_unknown(self):
        original = {"type": "picture",
                    "renditions": {
                        "original": {'CropLeft': 0, 'CropRight': 800, 'CropTop': 0, 'CropBottom': 600}
                    }
                    }
        doc = {'renditions': {'d': {'CropLeft': 0, 'CropRight': 800, 'CropTop': 0, 'CropBottom': 600}}}
        with self.assertRaises(SuperdeskApiError) as context:
            self.service.validate_crop(original, doc, "d")

        ex = context.exception
        self.assertEqual(ex.message, 'Unknown crop name! (name=d)')
        self.assertEqual(ex.status_code, 400)

    def test_add_crop_raises_error_if_original_missing(self):
        original = {
            'renditions': {
                '4-3': {
                }
            }
        }
        doc = {'CropLeft': 0, 'CropRight': 800, 'CropTop': 0, 'CropBottom': 600}
        with self.assertRaises(SuperdeskApiError) as context:
            self.service.create_crop(original, '4-3', doc)

        ex = context.exception
        self.assertEqual(ex.message, 'Original file couldn\'t be found')
        self.assertEqual(ex.status_code, 400)

    def test_validate_crop_converts_to_int(self):
        crop = {'width': '300', 'height': 200}
        self.service._validate_values(crop)
        self.assertEqual(300, crop['width'])
        self.assertEqual(200, crop['height'])

        with self.assertRaises(SuperdeskApiError) as context:
            self.service._validate_values({'width': 'foo'})
            self.assertEqual(context.exception.message, 'Invalid value for width in renditions')

    @mock.patch('superdesk.media.crop.crop_image', return_value=(False, 'test'))
    def test_add_crop_raises_error(self, crop_name):
        original = {
            'renditions': {
                'original': {
                }
            }
        }

        media = mock.MagicMock()
        media.name = 'test.jpg'

        with mock.patch('superdesk.app.media.get', return_value=media):
            doc = {'CropLeft': 0, 'CropRight': 800, 'CropTop': 0, 'CropBottom': 600}
            with self.assertRaises(SuperdeskApiError) as context:
                self.service.create_crop(original, '4-3', doc)

            ex = context.exception
            self.assertEqual(ex.message, 'Saving crop failed.')
            self.assertEqual(ex.status_code, 400)

    def test_crop_image_exact_size(self):
        img = get_picture_fixture()
        size = {'width': '300', 'height': '200'}
        crop = {'CropTop': '0', 'CropRight': '300', 'CropBottom': '200', 'CropLeft': '0'}
        with open(img, 'rb') as imgfile:
            res = crop_image(imgfile, img, crop, size)
            self.assertTrue(res[0])
            self.assertEqual(300, res[1].width)
            self.assertEqual(200, res[1].height)

    def test_resize_image(self):
        img = get_picture_fixture()
        with open(img, 'rb') as imgfile:
            resized, width, height = _resize_image(imgfile, ('200', None), 'jpeg')
            self.assertEqual(150, height)

    def test_get_rendition_spec_no_custom_crop(self):
        renditions = get_renditions_spec(no_custom_crops=True)
        for crop in self.crop_sizes.get('items'):
            self.assertNotIn(crop['name'], renditions)

    def test_get_rendition_spec_with_custom_crop(self):
        renditions = get_renditions_spec()
        for crop in self.crop_sizes.get('items'):
            self.assertIn(crop['name'], renditions)

    def test_can_generate_custom_crop_from_original(self):
        self.assertEquals(True, can_generate_custom_crop_from_original(800, 600, {'ratio': '16:9'}))
        self.assertEquals(True, can_generate_custom_crop_from_original(800, 600, {'width': 800, 'height': 600}))
        self.assertEquals(True, can_generate_custom_crop_from_original(810, 600, {'width': 800, 'height': 600}))
        self.assertEquals(True, can_generate_custom_crop_from_original(810, 610, {'width': 800, 'height': 600}))
        self.assertEquals(False, can_generate_custom_crop_from_original(780, 610, {'width': 800, 'height': 600}))
        self.assertEquals(False, can_generate_custom_crop_from_original(780, 590, {'width': 800, 'height': 600}))
        self.assertEquals(True, can_generate_custom_crop_from_original(780, 590, {'width': 800}))
        self.assertEquals(True, can_generate_custom_crop_from_original(780, 590, {'height': 800}))
        self.assertEquals(False, can_generate_custom_crop_from_original(780, 590, None))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


from copy import copy

from eve.utils import ParsedRequest
from eve.versioning import resolve_document_version

import superdesk
from superdesk.users.services import current_user_has_privilege
from superdesk.resource import Resource
from superdesk.errors import SuperdeskApiError, InvalidStateTransitionError
from superdesk.notification import push_notification
from superdesk.utc import utcnow
from superdesk.metadata.utils import item_url
from superdesk.services import BaseService
from superdesk.metadata.item import metadata_schema, ITEM_STATE, CONTENT_STATE, ITEM_TYPE
from superdesk import get_resource_service, config
from superdesk.activity import add_activity, ACTIVITY_CREATE, ACTIVITY_UPDATE
from superdesk.workflow import is_workflow_state_transition_valid
from apps.archive.common import get_item_expiry, item_operations, ITEM_OPERATION, update_version, \
    insert_into_versions, is_assigned_to_a_desk, convert_task_attributes_to_objectId, on_create_item, \
    ARCHIVE, get_subject
from flask_babel import _

task_statuses = ['todo', 'in_progress', 'done']
default_status = 'todo'
ITEM_SEND = 'send'
item_operations.append(ITEM_SEND)
MACRO_INCOMING = 'incoming'
MACRO_OUTGOING = 'outgoing'
MACRO_ONSTAGE = 'onstage'


def init_app(app):
    endpoint_name = 'tasks'
    service = TasksService(TaskResource.datasource['source'], backend=superdesk.get_backend())
    TaskResource(endpoint_name, app=app, service=service)


def compare_dictionaries(original, updates):
    original_keys = set(original.keys())
    updates_keys = set(updates.keys())
    intersect_keys = original_keys.intersection(updates_keys)
    added = list(updates_keys - original_keys)
    modified = [o for o in intersect_keys if original[o] != updates[o]]
    modified.extend(added)
    return modified


def send_to(doc, update=None, desk_id=None, stage_id=None, user_id=None, default_stage='incoming_stage'):
    """Send item to given desk and stage.

    Applies the outgoing and incoming macros of current and destination stages

    :param doc: original document to be sent
    :param update: updates for the document
    :param desk: id of desk where item should be sent
    :param stage: optional stage within the desk
    :param default_stage: if no stage_id is passed then it determines the stage in that desk the doc is assigned,
            either the the incomming stage or the working stage.
    """

    original_task = doc.setdefault('task', {})
    current_stage = None
    if original_task.get('stage'):
        current_stage = get_resource_service('stages').find_one(req=None, _id=original_task.get('stage'))
    desk = destination_stage = None
    task = {'desk': desk_id, 'stage': stage_id, 'user': original_task.get('user') if user_id is None else user_id}

    if current_stage:
        apply_stage_rule(doc, update, current_stage, MACRO_OUTGOING)

    if desk_id:
        desk = superdesk.get_resource_service('desks').find_one(req=None, _id=desk_id)
        if not desk:
            raise SuperdeskApiError.notFoundError(_('Invalid desk identifier {desk_id}').format(desk_id=desk_id))

        if not current_user_has_privilege('move') and \
                str(user_id) not in [str(x.get('user', '')) for x in desk.get('members', [])]:
            raise SuperdeskApiError.forbiddenError(_('User is not member of desk: {desk_id}').format(desk_id=desk_id))

        task['desk'] = desk_id
        if not stage_id:
            task['stage'] = desk.get(default_stage)
            destination_stage = get_resource_service('stages').find_one(req=None, _id=desk.get(default_stage))

    if stage_id:
        destination_stage = get_resource_service('stages').find_one(req=None, _id=stage_id)
        if not destination_stage:
            raise SuperdeskApiError.notFoundError(_('Invalid stage identifier {stage_id}').format(stage_id=stage_id))

        task['desk'] = destination_stage['desk']
        task['stage'] = stage_id

    if destination_stage:
        apply_stage_rule(doc, update, destination_stage, MACRO_INCOMING, desk=desk, task=task)
        if destination_stage.get('task_status'):
            task['status'] = destination_stage['task_status']

    if update:
        update.setdefault('task', {})
        update['task'].update(task)
        update['expiry'] = get_item_expiry(desk=desk, stage=destination_stage)
    else:
        doc['task'].update(task)
        doc['expiry'] = get_item_expiry(desk=desk, stage=destination_stage)
        superdesk.get_resource_service('desks').apply_desk_metadata(doc, doc)


def apply_stage_rule(doc, update, stage, rule_type, desk=None, task=None):
    macro_type = '{}_macro'.format(rule_type)

    if stage.get(macro_type):
        try:
            original_doc = dict(doc)
            macro = get_resource_service('macros').get_macro_by_name(stage.get(macro_type))
            macro['callback'](doc, desk=desk, stage=stage, task=task)
            if update:
                modified = compare_dictionaries(original_doc, doc)
                for i in modified:
                    update[i] = doc[i]
        except Exception as ex:
            message = _('Error:{exception} in {rule_type} rule:{rule} for stage:{stage}').format(
                exception=str(ex), rule_type=rule_type, rule=macro.get('label'), stage=stage.get('name'))
            raise SuperdeskApiError.badRequestError(message)


def apply_onstage_rule(doc, _id):
    """Apply any on stage macro/rule that may be defined for the stage.

    :param doc:
    :param _id:
    :return:
    """
    doc[config.ID_FIELD] = _id
    stage = get_resource_service('stages').find_one(req=None, _id=doc.get('task', {}).get('stage'))
    if stage:
        apply_stage_rule(doc, None, stage, 'onstage')


class TaskResource(Resource):
    datasource = {
        'source': 'archive',
        'default_sort': [('_updated', -1)],
        'filter': {'task': {'$exists': True}},
        'elastic_filter': {'bool': {
            'must': {'exists': {'field': 'task'}},
            'must_not': {'term': {ITEM_STATE: 'spiked'}},
        }}
    }

    item_url = item_url
    schema = {
        'slugline': metadata_schema['slugline'],
        'description_text': metadata_schema['description_text'],
        'type': metadata_schema['type'],
        'task': {
            'type': 'dict',
            'schema': {
                'status': {
                    'type': 'string',
                    'allowed': task_statuses,
                    'default': default_status
                },
                'due_date': {'type': 'datetime'},
                'started_at': {'type': 'datetime'},
                'finished_at': {'type': 'datetime'},
                'user': Resource.rel('users', True),
                'desk': Resource.rel('desks', True),
                'stage': Resource.rel('stages', True)
            }
        }
    }
    privileges = {'POST': 'tasks', 'PATCH': 'tasks', 'DELETE': 'tasks'}


class TasksService(BaseService):

    def get(self, req, lookup):
        if req is None:
            req = ParsedRequest()
        return self.backend.get('tasks', req=req, lookup=lookup)

    def update_times(self, doc):
        task = doc.get('task', {})
        status = task.get('status', None)
        if status == 'in_progress':
            task.setdefault('started_at', utcnow())

        if status == 'done':
            task.setdefault('finished_at', utcnow())

    def __is_content_moved_from_desk(self, doc):
        """
        Returns True if the 'doc' is being moved from a desk. False otherwise.
        """
        return doc.get('task', {}).get('desk', None) is None

    def __is_content_assigned_to_new_desk(self, original, updates):
        """Checks if the content is assigned to a new desk.

        :return: True if the content is being moved to a new desk. False otherwise.
        """
        return str(original.get('task', {}).get('desk', '')) != str(updates.get('task', {}).get('desk', ''))

    def __update_state(self, updates, original):
        if self.__is_content_assigned_to_new_desk(original, updates):
            # check if the preconditions for the action are in place
            original_state = original[ITEM_STATE]
            if not is_workflow_state_transition_valid('move', original_state):
                raise InvalidStateTransitionError()

            updates[ITEM_STATE] = CONTENT_STATE.DRAFT if self.__is_content_moved_from_desk(updates) \
                else CONTENT_STATE.SUBMITTED
            resolve_document_version(updates, ARCHIVE, 'PATCH', original)

    def update_stage(self, doc):
        task = doc.get('task', {})
        desk_id = task.get('desk', None)
        stage_id = task.get('stage', None)
        send_to(doc=doc, desk_id=desk_id, stage_id=stage_id)

    def on_create(self, docs):
        on_create_item(docs)
        for doc in docs:
            resolve_document_version(doc, ARCHIVE, 'POST')
            self.update_times(doc)
            self.update_stage(doc)
            convert_task_attributes_to_objectId(doc)

    def on_created(self, docs):
        push_notification(self.datasource, created=1)
        push_notification('task:new')
        for doc in docs:
            insert_into_versions(doc['_id'])
            if is_assigned_to_a_desk(doc):
                add_activity(ACTIVITY_CREATE, 'added new task {{ subject }} of type {{ type }}',
                             self.datasource, item=doc,
                             subject=get_subject(doc), type=doc[ITEM_TYPE])

    def on_update(self, updates, original):
        self.update_times(updates)
        if is_assigned_to_a_desk(updates):
            self.__update_state(updates, original)
        new_stage_id = str(updates.get('task', {}).get('stage', ''))
        old_stage_id = str(original.get('task', {}).get('stage', ''))
        new_user_id = updates.get('task', {}).get('user', '')
        if new_stage_id and new_stage_id != old_stage_id:
            updates[ITEM_OPERATION] = ITEM_SEND
            send_to(doc=original, update=updates, desk_id=None, stage_id=new_stage_id, user_id=new_user_id)
            resolve_document_version(updates, ARCHIVE, 'PATCH', original)
        convert_task_attributes_to_objectId(updates)
        update_version(updates, original)

    def on_updated(self, updates, original):
        updated = copy(original)
        updated.update(updates)
        if self._stage_changed(updates, original):
            insert_into_versions(doc=updated)
        new_task = updates.get('task', {})
        old_task = original.get('task', {})
        if new_task.get('stage') != old_task.get('stage'):
            push_notification('task:stage',
                              new_stage=str(new_task.get('stage', '')),
                              old_stage=str(old_task.get('stage', '')),
                              new_desk=str(new_task.get('desk', '')),
                              old_desk=str(old_task.get('desk', ''))
                              )
        else:
            push_notification(self.datasource, updated=1)

        if is_assigned_to_a_desk(updated):
            if self.__is_content_assigned_to_new_desk(original, updates) and \
                    not self._stage_changed(updates, original):
                insert_into_versions(doc=updated)
            add_activity(ACTIVITY_UPDATE, 'updated task {{ subject }} for item {{ type }}',
                         self.datasource, item=updated, subject=get_subject(updated), type=updated['type'])

    def on_deleted(self, doc):
        push_notification(self.datasource, deleted=1)

    def assign_user(self, item_id, user):
        return self.patch(item_id, {'task': user})

    def _stage_changed(self, updates, original):
        new_stage_id = str(updates.get('task', {}).get('stage', ''))
        old_stage_id = str(original.get('task', {}).get('stage', ''))
        return new_stage_id and new_stage_id != old_stage_id


superdesk.privilege(name='tasks', label='Tasks Management', description='Tasks Management')



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

"""Download module"""
import logging
import superdesk
from superdesk.errors import SuperdeskApiError
from werkzeug.wsgi import wrap_file
from .resource import Resource
from .services import BaseService
from flask import url_for, request, current_app as app

bp = superdesk.Blueprint('download_raw', __name__)
logger = logging.getLogger(__name__)


@bp.route('/download/<id>', methods=['GET'], defaults={'folder': None})
@bp.route('/download/<path:folder>/<id>', methods=['GET'])
def download_file(id, folder=None):
    filename = '{}/{}'.format(folder, id) if folder else id

    file = app.media.get(filename, 'download')
    if file:
        data = wrap_file(request.environ, file, buffer_size=1024 * 256)
        response = app.response_class(
            data,
            mimetype=file.content_type,
            direct_passthrough=True)
        response.content_length = file.length
        response.last_modified = file.upload_date
        response.headers['Content-Disposition'] = 'attachment; filename=\"export.zip\"'
        return response
    raise SuperdeskApiError.notFoundError('File not found on media storage.')


def download_url(media_id):
    return url_for('download_raw.download_file', id=media_id, _external=True)


def init_app(app):
    endpoint_name = 'download'
    app.download_url = download_url
    superdesk.blueprint(bp, app)
    service = BaseService(endpoint_name, backend=superdesk.get_backend())
    DownloadResource(endpoint_name, app=app, service=service)


class DownloadResource(Resource):
    schema = {
        'file': {'type': 'file'}
    }
    item_methods = []



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
from apps.auth import get_user
from eve.versioning import resolve_document_version
from flask import request, current_app as app
from apps.archive import ArchiveSpikeService
from superdesk import get_resource_service, Service, config
from superdesk.metadata.item import ITEM_STATE, EMBARGO, CONTENT_STATE, CONTENT_TYPE, \
    ITEM_TYPE, PUBLISH_STATES, ASSOCIATIONS, GUID_TAG, PROCESSED_FROM
from superdesk.resource import Resource, build_custom_hateoas
from apps.archive.common import CUSTOM_HATEOAS, ITEM_CREATE, ARCHIVE, BROADCAST_GENRE, ITEM_REWRITE, \
    ITEM_UNLINK, ITEM_LINK, insert_into_versions
from superdesk.metadata.utils import item_url, generate_guid
from superdesk.workflow import is_workflow_state_transition_valid
from superdesk.errors import SuperdeskApiError, InvalidStateTransitionError
from superdesk.notification import push_notification
from superdesk.signals import item_rewrite
from apps.tasks import send_to
from apps.archive.archive import update_associations
from flask_babel import _

logger = logging.getLogger(__name__)


class ArchiveRewriteResource(Resource):
    endpoint_name = 'archive_rewrite'
    resource_title = endpoint_name

    schema = {
        'desk_id': {'type': 'string', 'nullable': True},
        'update': {'type': 'dict', 'nullable': True}
    }

    url = 'archive/<{0}:original_id>/rewrite'.format(item_url)
    resource_methods = ['POST', 'DELETE']
    privileges = {'POST': 'rewrite', 'DELETE': 'rewrite'}


class ArchiveRewriteService(Service):
    def create(self, docs, **kwargs):
        doc = docs[0] if len(docs) > 0 else {}
        original_id = request.view_args['original_id']
        update_document = doc.get('update')

        archive_service = get_resource_service(ARCHIVE)
        original = archive_service.find_one(req=None, _id=original_id)
        self._validate_rewrite(original, update_document)

        rewrite = self._create_rewrite_article(original,
                                               existing_item=update_document,
                                               desk_id=doc.get('desk_id'))

        if 'body_html' in rewrite:
            if 'fields_meta' in original:
                rewrite['fields_meta'] = original['fields_meta']
            update_associations(rewrite)

        # signal
        item_rewrite.send(self, item=rewrite, original=original)

        if update_document:
            # process the existing story
            archive_service.patch(update_document[config.ID_FIELD], rewrite)
            app.on_archive_item_updated(rewrite, update_document, ITEM_LINK)
            rewrite[config.ID_FIELD] = update_document[config.ID_FIELD]
            ids = [update_document[config.ID_FIELD]]
        else:
            # Set the version.
            resolve_document_version(rewrite, ARCHIVE, "POST")
            ids = archive_service.post([rewrite])
            insert_into_versions(doc=rewrite)
            build_custom_hateoas(CUSTOM_HATEOAS, rewrite)

            app.on_archive_item_updated({'rewrite_of': rewrite.get('rewrite_of')}, rewrite, ITEM_LINK)

        self._add_rewritten_flag(original, rewrite)
        get_resource_service('archive_broadcast').on_broadcast_master_updated(ITEM_CREATE,
                                                                              item=original,
                                                                              rewrite_id=ids[0])

        doc.clear()
        doc.update(rewrite)
        return ids

    def _validate_rewrite(self, original, update):
        """Validates the article to be rewritten.

        :param original: article to be rewritten
        :param update: article as the rewrite
        :raises: SuperdeskApiError
        """
        if not original:
            raise SuperdeskApiError.notFoundError(message=_('Cannot find the article'))

        if original.get(EMBARGO):
            raise SuperdeskApiError.badRequestError(_("Rewrite of an Item having embargo isn't possible"))

        if not original.get('event_id'):
            raise SuperdeskApiError.notFoundError(message=_('Event id does not exist'))

        if original.get('rewritten_by'):
            raise SuperdeskApiError.badRequestError(message=_('Article has been rewritten before !'))

        if not is_workflow_state_transition_valid('rewrite', original[ITEM_STATE]):
            raise InvalidStateTransitionError()

        if original.get('rewrite_of') and not (original.get(ITEM_STATE) in PUBLISH_STATES):
            raise SuperdeskApiError.badRequestError(
                message=_("Rewrite is not published. Cannot rewrite the story again."))

        if update:
            # in case of associate as update
            if update.get('rewrite_of'):
                raise SuperdeskApiError.badRequestError(_("Rewrite story has been used as update before !"))

            if update.get(ITEM_STATE) in [CONTENT_STATE.PUBLISHED,
                                          CONTENT_STATE.CORRECTED,
                                          CONTENT_STATE.KILLED,
                                          CONTENT_STATE.RECALLED,
                                          CONTENT_STATE.SCHEDULED,
                                          CONTENT_STATE.SPIKED]:
                raise InvalidStateTransitionError()

            if update.get(ITEM_TYPE) not in [CONTENT_TYPE.TEXT, CONTENT_TYPE.PREFORMATTED]:
                raise SuperdeskApiError.badRequestError(_("Rewrite story can only be text or pre-formatted !"))

            if update.get('genre') and \
                    any(genre.get('value', '').lower() == BROADCAST_GENRE.lower() for genre in update.get('genre')):
                raise SuperdeskApiError.badRequestError(_("Broadcast cannot be a update story !"))

            if original.get('profile') and original.get('profile') != update.get('profile'):
                raise SuperdeskApiError.badRequestError(_("Rewrite item content profile does "
                                                        "not match with Original item."))

    def _create_rewrite_article(self, original, existing_item=None, desk_id=None):
        """Creates a new story and sets the metadata from original.

        :param dict original: original story
        :param dict existing_item: existing story that is being re-written
        :return:new story
        """
        rewrite = dict()

        fields = ['family_id', 'event_id', 'flags', 'language', ASSOCIATIONS, 'extra']
        existing_item_preserve_fields = (ASSOCIATIONS, 'flags')

        if app.config.get('COPY_ON_REWRITE_FIELDS'):
            fields.extend(app.config['COPY_ON_REWRITE_FIELDS'])

        if existing_item:
            # for associate an existing file as update merge subjects
            subjects = original.get('subject', [])
            unique_subjects = {subject.get('qcode') for subject in subjects}
            rewrite['subject'] = [subject for subject in existing_item.get('subject', [])
                                  if subject.get('qcode') not in unique_subjects]
            rewrite['subject'].extend(subjects)
            rewrite['flags'] = original['flags'] or {}

            # preserve flags
            for key in rewrite.get('flags').keys():
                rewrite['flags'][key] = original['flags'][key] or existing_item.get('flags', {}).get(key, False)

            original_associations = original.get(ASSOCIATIONS) or {}
            existing_associations = existing_item.get(ASSOCIATIONS) or {}
            rewrite[ASSOCIATIONS] = existing_associations

            # if the existing item has association then preserve the association
            for key, assoc in original_associations.items():
                if not existing_associations.get(key):
                    rewrite[ASSOCIATIONS][key] = assoc
        else:
            # ingest provider and source to be retained for new item
            fields.extend(['ingest_provider', 'source'])

            if original.get('profile'):
                content_type = get_resource_service('content_types').find_one(req=None, _id=original['profile'])
                extended_fields = list(content_type['schema'].keys())
                # extra fields needed.
                extended_fields.extend(['profile', 'keywords', 'target_regions',
                                        'target_types', 'target_subscribers'])
            else:
                extended_fields = [
                    'abstract', 'anpa_category', 'pubstatus', 'slugline', 'urgency',
                    'subject', 'priority', 'byline', 'dateline', 'headline', 'place',
                    'genre', 'body_footer', 'company_codes', 'keywords',
                    'target_regions', 'target_types', 'target_subscribers'
                ]

            fields.extend(extended_fields)

        for field in fields:
            if original.get(field):
                # don't overwrite some fields in existing items
                if existing_item and field in existing_item_preserve_fields:
                    continue

                rewrite[field] = original[field]

        # if the original was flagged for SMS the rewrite should not be.
        if not existing_item and rewrite.get('flags', {}).get('marked_for_sms', False):
            rewrite['flags']['marked_for_sms'] = False

        # SD-4595 - Default value for the update article to be set based on the system config.
        if config.RESET_PRIORITY_VALUE_FOR_UPDATE_ARTICLES:
            # if True then reset to the default priority value.
            rewrite['priority'] = int(config.DEFAULT_PRIORITY_VALUE_FOR_MANUAL_ARTICLES)

        rewrite['rewrite_of'] = original[config.ID_FIELD]
        rewrite['rewrite_sequence'] = (original.get('rewrite_sequence') or 0) + 1
        rewrite.pop(PROCESSED_FROM, None)

        if not existing_item:
            # send the document to the desk only if a new rewrite is created
            send_to(doc=rewrite, desk_id=(desk_id or original['task']['desk']), default_stage='working_stage')

            # if we are rewriting a published item then copy the body_html
            if original.get('state', '') in (CONTENT_STATE.PUBLISHED, CONTENT_STATE.CORRECTED):
                rewrite['body_html'] = original.get('body_html', '')

        rewrite[ITEM_STATE] = CONTENT_STATE.PROGRESS
        self._set_take_key(rewrite)
        return rewrite

    def _add_rewritten_flag(self, original, rewrite):
        """Adds rewritten_by field to the existing published items.

        :param dict original: item on which rewrite is triggered
        :param dict rewrite: rewritten document
        """
        get_resource_service('published').update_published_items(original[config.ID_FIELD],
                                                                 'rewritten_by', rewrite[config.ID_FIELD])

        # modify the original item as well.
        get_resource_service(ARCHIVE).system_update(original[config.ID_FIELD],
                                                    {'rewritten_by': rewrite[config.ID_FIELD]}, original)
        app.on_archive_item_updated({'rewritten_by': rewrite[config.ID_FIELD]}, original, ITEM_REWRITE)

    def _set_take_key(self, rewrite):
        """Sets the anpa take key of the rewrite with ordinal.

        :param rewrite: rewrite story
        """
        rewrite_sequence = rewrite.get('rewrite_sequence') or 0
        if rewrite_sequence > 1:
            ordinal = self._get_ordinal(rewrite_sequence)
            rewrite['anpa_take_key'] = '{} update'.format(ordinal)
        else:
            rewrite['anpa_take_key'] = 'update'

    def _get_ordinal(self, n):
        """Returns the ordinal value of the given int."""
        if 10 <= n % 100 < 20:
            return str(n) + 'th'
        else:
            return str(n) + {1: 'st', 2: 'nd', 3: 'rd'}.get(n % 10, "th")

    def delete(self, lookup):
        target_id = request.view_args['original_id']
        archive_service = get_resource_service(ARCHIVE)
        target = archive_service.find_one(req=None, _id=target_id)
        updates = {}

        if target.get('rewrite_of'):
            # remove the rewrite info
            ArchiveSpikeService().update_rewrite(target)

        if not target.get('rewrite_of'):
            # there is nothing to do
            raise SuperdeskApiError.badRequestError(_("Only updates can be unlinked!"))

        if target.get('rewrite_of'):
            updates['rewrite_of'] = None

        if target.get('anpa_take_key'):
            updates['anpa_take_key'] = None

        if target.get('rewrite_sequence'):
            updates['rewrite_sequence'] = None

        if target.get('sequence'):
            updates['sequence'] = None

        updates['event_id'] = generate_guid(type=GUID_TAG)

        archive_service.system_update(target_id, updates, target)
        user = get_user(required=True)
        push_notification('item:unlink', item=target_id, user=str(user.get(config.ID_FIELD)))
        app.on_archive_item_updated(updates, target, ITEM_UNLINK)




import superdesk

from superdesk.notification import push_notification
from apps.content import push_content_notification
from superdesk import get_resource_service

PRIVILEGE = 'desk_routing'


class ClosedDeskResource(superdesk.Resource):
    """Custom resource to change desk.is_closed

    Allowing users without desk management privileges to close/open desk.
    """

    schema = {
        'is_closed': {'type': 'boolean'},
        'closed_destination': superdesk.Resource.rel('desks'),
    }

    datasource = {
        'source': 'desks',
    }

    privileges = {
        'PATCH': PRIVILEGE,  # anyone who can edit can turn it on/off
    }

    resource_methods = []
    item_methods = ['PATCH']


class ClosedDeskService(superdesk.Service):

    def on_updated(self, updates, original):
        is_closed = updates.get('is_closed', False)
        if not is_closed and original.get('is_closed'):
            desk_id = updates['_id']
            self.remove_marks(desk_id)
        push_notification('desks:closed',
                          is_closed=is_closed,
                          _id=original.get('_id'),
                          _etag=updates.get('_etag'))

    def remove_marks(self, desk_id):
        """Remove "mark for desk" attribute

        :param ObjectId desk_id: id of the desk being re-opened
        """
        for service_name in ('archive', 'published'):
            service = get_resource_service(service_name)
            marked_items = service.find({'task.desk': desk_id, 'marked_desks': {'$exists': True, '$ne': []}})
            for item in marked_items:
                marked_desks = [m for m in item['marked_desks'] if 'user_marked' in m]
                service.system_update(item['_id'], {'marked_desks': marked_desks}, item)
                push_content_notification([item])


def init_app(app):
    app.config['DOMAIN']['desks']['schema'].update({
        'is_closed': {'type': 'boolean'},
        'closed_destination': superdesk.Resource.rel('desks'),
    })

    app.config['SOURCES']['desks']['projection'].update({
        'is_closed': 1,
        'closed_destination': 1,
    })

    superdesk.register_resource('closed_desks', ClosedDeskResource, ClosedDeskService, _app=app)

    superdesk.privilege(
        name=PRIVILEGE,
        label='Desk routing',
        description='User can configure desk routing')



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import superdesk

from superdesk.errors import SuperdeskApiError
from superdesk.utils import ListCursor
from .macro_register import macros


def get_public_props(item):
    return {k: v for k, v in item.items() if k != 'callback'}


class MacrosService(superdesk.Service):

    def get(self, req, lookup):
        """Return all registered macros."""
        desk = getattr(req, 'args', {}).get('desk')
        include_backend = getattr(req, 'args', {}).get('backend') == 'true'
        all_macros = self.get_macros(include_backend)

        if desk:
            return ListCursor([get_public_props(macro) for macro in all_macros if
                               desk.upper() in macro.get('desks', []) or macro.get('desks') is None])
        else:
            return ListCursor([get_public_props(macro) for macro in all_macros])

    def create(self, docs, **kwargs):
        try:
            ids = []
            for doc in docs:
                res = self.execute_macro(doc['item'], doc['macro'])
                if isinstance(res, tuple):
                    doc['item'] = res[0]
                    doc['diff'] = res[1]
                else:
                    doc['item'] = res
                if doc.get('commit'):
                    item = superdesk.get_resource_service('archive').find_one(req=None, _id=doc['item']['_id'])
                    updates = doc['item'].copy()
                    updates.pop('_id')
                    superdesk.get_resource_service('archive').update(item['_id'], updates, item)
                ids.append(doc['macro'])
            return ids
        except Exception as ex:
            raise SuperdeskApiError.internalError(str(ex), exception=ex)

    def get_macro_by_name(self, macro_name):
        return macros.find(macro_name)

    def execute_macro(self, doc, macro_name):
        macro = self.get_macro_by_name(macro_name)
        return macro['callback'](doc)

    def get_macros(self, include_backend):
        if include_backend:
            return macros
        else:
            return [m for m in macros if m.get('access_type') == 'frontend']

    def execute_translation_macro(self, doc, from_language, to_language):
        """
        Apply to doc all macros that are related to current translation defined by from_language and to_language
        Macros can have optionally defined translation related settings: from_languages, to_languages.
        """
        for m in macros:
            if (not m.get('from_languages', None) or from_language in m['from_languages']) \
                    and to_language in m.get('to_languages', []):
                m['callback'](doc, from_language=from_language, to_language=to_language)


class MacrosResource(superdesk.Resource):
    resource_methods = ['GET', 'POST']
    item_methods = []
    privileges = {'POST': 'archive'}

    schema = {
        'macro': {
            'type': 'string',
            'required': True,
            'allowed': macros
        },
        'item': {
            'type': 'dict',
        },
        'commit': {
            'type': 'boolean',
            'default': False,
        },
        'diff': {
            'type': 'dict',
            'readonly': True,
        },
    }



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2019 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk
from os.path import expanduser, abspath, exists

import json


class GenerateVocabularies(superdesk.Command):
    """Generate vocabularies.json from flat file

    This command generate the vocabularies.json file from a flat text file having the following structure:
    ::

        Vocabulary Label:
            item 1
            item 2
            item 3

    Vocabulary label must end by a colon (:). Indendation is not significant.
    By default vocabularies maps to an id with the label in lower case where spaces
    have been replaced by underscode (_).
    It is possible though to map to an other label/id. For this, you can specify a json
    file using the ``--key-map`` option. It expects a path to json file mapping from vocabulary label
    set in the flat text file to superdesk id (which will also be used as label).
    e.g.:

    .. code:: json

        {
            "author types": "author roles",
            "news statuses": null,
            "news urgencies": "urgency",
            "genres": {
                "name": "Genre",
                "extra": {"selection_type": "multi selection"},
            },
            "": {
                "extra": {"type": "unmanageable"}
            }
        }

    - plain text name will be used as label/id.
    - null means that the value must be ignored
    - a dict helps to manage more complicated mapping. See below for the keys which can be used.
    - empty string ("") can be used as key when extra must be applied to all non ignored vocabularies

    Key which can be used in dictionary:

    =====  ===========
    key    explanation
    =====  ===========
    name   used as label and _id (in lower case and with spaces replaced by underscodes)
    extra  a dictionary which will be merged to default one. Useful to add missing keys/values.
    =====  ===========

    You can also specify a path to a file containing a json array using ``--base`` which will contain
    the base vocabularies to use. If not specified, an empty array will be used.

    The generated json will be written to ``vocabularies.json`` in the current directory.
    """

    option_list = [
        superdesk.Option('-k', '--keys-map', help='key mapping json file'),
        superdesk.Option('-b', '--base', help='json array to use as base vocabularies'),
        superdesk.Option('-f', '--force', action="store_true",
                         help='overwritte "vocabularies.json" if it already exists'),
        superdesk.Option('source_file', help='plain text file with the vocabularies to create')
    ]

    def get_path(self, path):
        return abspath(expanduser(path))

    def run(self, keys_map, base, force, source_file):
        if exists("vocabularies.json"):
            if force:
                print('Overwriting "vocabularies.json" as requested')
            else:
                raise SystemExit('"vocabularies.json" file already exists, won\'t overwrite')
        if base is None:
            voc = []
        else:
            voc = json.load(open(self.get_path(base)))
            if not isinstance(voc, list):
                raise SystemExit('base vocabularies must be an array')

        if keys_map is None:
            keys_map = {}
        else:
            keys_map = {k.lower(): v for k, v in json.load(open(self.get_path(keys_map))).items()}

        base_extra = keys_map.pop("", {}).get("extra")

        with open(self.get_path(source_file)) as f:
            lines = f.readlines()

        skip = False
        current_voc = None
        for line in lines:
            line = line.strip()
            if not line:
                continue
            if line.endswith(':'):
                skip = False
                name = line[:-1].strip()
                extra = None
                if name.lower() in keys_map:
                    name = keys_map[name.lower()]
                    if name is None:
                        skip = True
                        continue
                    if isinstance(name, dict):
                        extra = name.get(u'extra')
                        name = name.get(u'name')
                current_voc = {
                    '_id': name.lower().replace(' ', '_'),
                    "display_name": name,
                    "type": "manageable",
                    "unique_field": "qcode",
                    "service": {"all": 1},
                    'items': [],
                }
                if base_extra:
                    current_voc.update(base_extra)
                if extra:
                    current_voc.update(extra)
                voc.append(current_voc)
            elif not skip:
                if current_voc is None:
                    raise SystemExit(
                        'Invalid source file! Your file must start with a vocabularies label (it must end with a '
                        'colon)')
                item = {
                    u'name': line,
                    u'qcode': line,
                    u'is_active': True,
                }
                current_voc['items'].append(item)

        json.dump(voc, open("vocabularies.json", "w"), indent=4)
        print('Data generated in "vocabularies.json"')


superdesk.command('vocabularies:generate', GenerateVocabularies())




from flask import abort, request
from superdesk.resource import Resource
from superdesk.services import Service
from superdesk.metadata.utils import item_url
from superdesk.metadata.item import CONTENT_STATE
from .archive import ArchiveResource


ARCHIVE_DATASOURCE = ArchiveResource.datasource


def exclude_filter():
    if request:
        return {
            'bool': {
                'must_not': {
                    'term': {
                        '_id': request.view_args['item_id'],
                    }
                }
            }
        }


class ArchiveRelatedResource(Resource):
    endpoint_name = 'archive_related'
    url = 'archive/<{0}:item_id>/related'.format(item_url)
    schema = {'lock_action': {'type': 'string'}}
    datasource = {
        'source': 'archive',
        'projection': ARCHIVE_DATASOURCE['projection'],
        'default_sort': ARCHIVE_DATASOURCE['default_sort'],
        'elastic_filter': {'bool': {
            'must': {'terms': {'state': [
                CONTENT_STATE.INGESTED,
                CONTENT_STATE.ROUTED,
                CONTENT_STATE.FETCHED,
                CONTENT_STATE.SUBMITTED,
                CONTENT_STATE.PROGRESS,

                CONTENT_STATE.PUBLISHED,
                CONTENT_STATE.CORRECTED,
                CONTENT_STATE.SCHEDULED,

                CONTENT_STATE.KILLED,
                CONTENT_STATE.RECALLED,
                CONTENT_STATE.UNPUBLISHED,
            ]}},
        }},
        'elastic_filter_callback': exclude_filter,
    }
    resource_methods = ['GET']
    item_methods = []
    resource_title = endpoint_name


class ArchiveRelatedService(Service):
    def get(self, req, lookup):
        item = self.find_one(req=req, _id=lookup['item_id'])
        if not item or not item.get('family_id'):
            abort(404)
        return super().get(req, {'family_id': item['family_id']})



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import copy
import unittest
import superdesk

from datetime import timedelta, datetime
from unittest.mock import MagicMock, patch

from bson import ObjectId
from pytz import timezone

from apps.archive.archive import SOURCE as ARCHIVE, update_image_caption, update_associations
from superdesk.metadata.item import CONTENT_STATE
from apps.archive.common import (
    validate_schedule, remove_media_files,
    format_dateline_to_locmmmddsrc, convert_task_attributes_to_objectId,
    is_genre, BROADCAST_GENRE, get_default_source, set_default_source,
    get_utc_schedule, get_dateline_city, transtype_metadata
)
from superdesk import get_resource_service
from superdesk.errors import SuperdeskApiError
from superdesk.media.crop import CropService
from superdesk.tests import TestCase
from superdesk.utc import get_expiry_date, utcnow
from apps.publish.content import publish
from unittest import mock
from apps.search_providers import register_search_provider, registered_search_providers

NOW = utcnow()


class RemoveSpikedContentTestCase(TestCase):

    articles = [{'guid': 'tag:localhost:2015:69b961ab-2816-4b8a-a584-a7b402fed4f9',
                 '_id': '1',
                 'type': 'text',
                 'last_version': 3,
                 '_current_version': 4,
                 'body_html': 'Test body',
                 'urgency': 4,
                 'headline': 'Two students missing',
                 'pubstatus': 'usable',
                 'firstcreated': utcnow(),
                 'byline': 'By Alan Karben',
                 'ednote': 'Andrew Marwood contributed to this article',
                 'keywords': ['Student', 'Crime', 'Police', 'Missing'],
                 'subject':[{'qcode': '17004000', 'name': 'Statistics'},
                            {'qcode': '04001002', 'name': 'Weather'}],
                 'state': 'draft',
                 'expiry': utcnow() + timedelta(minutes=20),
                 'unique_name': '#1'},
                {'guid': 'tag:localhost:2015:69b961ab-2816-4b8a-a974-xy4532fe33f9',
                 '_id': '2',
                 'last_version': 3,
                 '_current_version': 4,
                 'body_html': 'Test body of the second article',
                 'urgency': 4,
                 'headline': 'Another two students missing',
                 'pubstatus': 'usable',
                 'firstcreated': utcnow(),
                 'byline': 'By Alan Karben',
                 'ednote': 'Andrew Marwood contributed to this article',
                 'keywords': ['Student', 'Crime', 'Police', 'Missing'],
                 'subject':[{'qcode': '17004000', 'name': 'Statistics'},
                            {'qcode': '04001002', 'name': 'Weather'}],
                 'expiry': utcnow() + timedelta(minutes=20),
                 'state': 'draft',
                 'type': 'text',
                 'unique_name': '#2'},
                {'guid': 'tag:localhost:2015:69b961ab-2816-4b8a-a584-a7b402fed4fa',
                 '_id': '3',
                 '_current_version': 4,
                 'body_html': 'Test body',
                 'urgency': 4,
                 'headline': 'Two students missing killed',
                 'pubstatus': 'usable',
                 'firstcreated': utcnow(),
                 'byline': 'By Alan Karben',
                 'ednote': 'Andrew Marwood contributed to this article killed',
                 'keywords': ['Student', 'Crime', 'Police', 'Missing'],
                 'subject':[{'qcode': '17004000', 'name': 'Statistics'},
                            {'qcode': '04001002', 'name': 'Weather'}],
                 'state': 'draft',
                 'expiry': utcnow() + timedelta(minutes=20),
                 'type': 'text',
                 'unique_name': '#3'},
                {'guid': 'tag:localhost:2015:69b961ab-2816-4b8a-a584-a7b402fed4fc',
                 '_id': '4',
                 '_current_version': 3,
                 'state': 'draft',
                 'type': 'composite',
                 'groups': [{'id': 'root', 'refs': [{'idRef': 'main'}], 'role': 'grpRole:NEP'},
                            {
                                'id': 'main',
                                'refs': [
                                    {
                                        'location': 'archive',
                                        'guid': '1',
                                        'residRef': '1',
                                        'type': 'text'
                                    },
                                    {
                                        'location': 'archive',
                                        'residRef': '2',
                                        'guid': '2',
                                        'type': 'text'
                                    }
                                ],
                                'role': 'grpRole:main'}],
                 'firstcreated': utcnow(),
                 'expiry': utcnow() + timedelta(minutes=20),
                 'unique_name': '#4'},
                {'guid': 'tag:localhost:2015:69b961ab-4b8a-a584-2816-a7b402fed4fc',
                 '_id': '5',
                 '_current_version': 3,
                 'state': 'draft',
                 'type': 'composite',
                 'groups': [{'id': 'root', 'refs': [{'idRef': 'main'}, {'idRef': 'story'}], 'role': 'grpRole:NEP'},
                            {
                                'id': 'main',
                                'refs': [
                                    {
                                        'location': 'archive',
                                        'guid': '1',
                                        'residRef': '1',
                                        'type': 'text'
                                    }
                                ],
                                'role': 'grpRole:main'},
                            {
                                'id': 'story',
                                'refs': [
                                    {
                                        'location': 'archive',
                                        'guid': '4',
                                        'residRef': '4',
                                        'type': 'composite'
                                    }
                                ],
                                'role': 'grpRole:story'}],
                 'firstcreated': utcnow(),
                 'expiry': utcnow() + timedelta(minutes=20),
                 'unique_name': '#5'}]

    media = {
        'viewImage': {
            'media': '1592730d582080f4e9fcc2fcf43aa357bda0ed19ffe314ee3248624cd4d4bc54',
            'mimetype': 'image/jpeg',
            'href': 'http://192.168.220.209/api/upload/abc/raw?_schema=http',
            'height': 452,
            'width': 640
        },
        'thumbnail': {
            'media': '52250b4f37da50ee663fdbff057a5f064479f8a8bbd24fb8fdc06135d3f807bb',
            'mimetype': 'image/jpeg',
            'href': 'http://192.168.220.209/api/upload/abc/raw?_schema=http',
            'height': 120,
            'width': 169
        },
        'baseImage': {
            'media': '7a608aa8f51432483918027dd06d0ef385b90702bfeba84ac4aec38ed1660b18',
            'mimetype': 'image/jpeg',
            'href': 'http://192.168.220.209/api/upload/abc/raw?_schema=http',
            'height': 990,
            'width': 1400
        },
        'original': {
            'media': 'stub.jpeg',
            'mimetype': 'image/jpeg',
            'href': 'http://192.168.220.209/api/upload/stub.jpeg/raw?_schema=http',
            'height': 2475,
            'width': 3500
        }
    }

    def test_query_getting_expired_content(self):
        now = utcnow()

        self.app.data.insert(ARCHIVE, [
            {'expiry': get_expiry_date(0), 'state': 'spiked'},
            {'expiry': get_expiry_date(10), 'state': 'spiked'},
            {'expiry': get_expiry_date(20), 'state': 'spiked'},
            {'expiry': get_expiry_date(30), 'state': 'spiked'},
            {'expiry': None, 'state': 'spiked'},
            {'unique_id': 97, 'state': 'spiked'},
            {'expiry': now - timedelta(minutes=10), 'state': 'spiked', 'unique_id': 100},
        ])

        expired_items = get_resource_service(ARCHIVE).get_expired_items(now)
        now = utcnow()
        for expired_items in get_resource_service(ARCHIVE).get_expired_items(now):
            self.assertEquals(1, len(expired_items))
            self.assertEquals(100, expired_items[0]['unique_id'])

    def test_remove_media_files_for_picture(self):
        item = {
            '_id': 'testimage',
            'type': 'picture',
            'renditions': self.media
        }

        original = item.copy()
        with patch.object(self.app.media, 'delete') as media_delete:
            CropService().update_media_references(item, original)
            references_service = get_resource_service('media_references')
            refs = references_service.get(req=None, lookup={'item_id': 'testimage'})
            self.assertEqual(refs.count(), 4)
            for ref in refs:
                self.assertEqual(ref.get('published'), False)
            CropService().update_media_references(item, original, True)
            refs = references_service.get(req=None, lookup={'item_id': 'testimage'})
            for ref in refs:
                self.assertEqual(ref.get('published'), True)

            remove_media_files(item)
            self.assertEqual(0, media_delete.call_count)

            item = {
                '_id': 'testimage2',
                'type': 'picture',
                'renditions': self.media
            }

            original = item.copy()
            CropService().update_media_references(item, original)
            references_service = get_resource_service('media_references')
            refs = references_service.get(req=None, lookup={'item_id': 'testimage2'})
            self.assertEqual(refs.count(), 4)
            for ref in refs:
                self.assertEqual(ref.get('published'), False)

            remove_media_files(item)
            self.assertEqual(0, media_delete.call_count)

            item = {
                '_id': 'testimage3',
                'type': 'picture',
                'renditions': {
                    'viewImage': {
                        'media': '123',
                        'mimetype': 'image/jpeg',
                        'href': 'http://192.168.220.209/api/upload/abc/raw?_schema=http',
                        'height': 452,
                        'width': 640
                    },
                    'thumbnail': {
                        'media': '456',
                        'mimetype': 'image/jpeg',
                        'href': 'http://192.168.220.209/api/upload/abc/raw?_schema=http',
                        'height': 120,
                        'width': 169
                    }
                }
            }

            original = item.copy()
            CropService().update_media_references(item, original)
            references_service = get_resource_service('media_references')
            refs = references_service.get(req=None, lookup={'item_id': 'testimage3'})
            self.assertEqual(refs.count(), 2)
            for ref in refs:
                self.assertEqual(ref.get('published'), False)

            remove_media_files(item)
            self.assertEqual(2, media_delete.call_count)
            for key, rendition in item.get('renditions').items():
                media_delete.assert_any_call(rendition['media'])

    def test_remove_media_files_for_picture_associations(self):
        item = {
            '_id': 'testimage',
            'type': 'text',
            'associations': {
                'featuremedia': {
                    '_id': '123',
                    'renditions': self.media
                },
                'featurevideo': {
                    '_id': '456',
                    'renditions': {
                        'viewImage': {
                            'media': 'testing_123',
                            'mimetype': 'image/jpeg',
                            'href': 'http://192.168.220.209/api/upload/abc/raw?_schema=http',
                            'height': 452,
                            'width': 640
                        },
                        'thumbnail': {
                            'media': 'testing_456',
                            'mimetype': 'image/jpeg',
                            'href': 'http://192.168.220.209/api/upload/abc/raw?_schema=http',
                            'height': 120,
                            'width': 169
                        }
                    }
                }
            }
        }

        original = item.copy()
        with patch.object(self.app.media, 'delete') as media_delete:
            CropService().update_media_references(item, original)
            references_service = get_resource_service('media_references')
            refs = references_service.get(req=None, lookup={'item_id': 'testimage'})
            self.assertEqual(refs.count(), 6)
            for ref in refs:
                self.assertEqual(ref.get('published'), False)
            CropService().update_media_references(item, original, True)
            refs = references_service.get(req=None, lookup={'item_id': 'testimage'})
            for ref in refs:
                self.assertEqual(ref.get('published'), True)

            remove_media_files(item)
            self.assertEqual(0, media_delete.call_count)

    def test_remove_media_files_for_attachments(self):
        attachments = self.app.data.insert('attachments', [{'media': 'foo'}])
        item = {
            '_id': 'test',
            'type': 'text',
            'attachments': [
                {'attachment': attachments[0]},
            ]
        }
        with patch.object(self.app.media, 'delete') as media_delete:
            remove_media_files(item)
        media_delete.assert_any_call('foo', 'attachments')

    def test_delete_by_ids(self):
        ids = self.app.data.insert(ARCHIVE, self.articles)
        archive_service = get_resource_service(ARCHIVE)
        archive_service.on_delete = MagicMock()
        archive_service.delete_by_article_ids(ids)
        self.assertTrue(self.app.data.mongo.is_empty(ARCHIVE))
        self.assertTrue(self.app.data.elastic.is_empty(ARCHIVE))
        self.assertEqual(len(self.articles), archive_service.on_delete.call_count)

    def test_remove_renditions_from_all_versions(self):
        renditions = copy.copy(self.media)

        ids = self.app.data.insert(ARCHIVE, [{
            'state': 'spiked',
            'expiry': get_expiry_date(-10),
            'type': 'picture',
            'renditions': {},
        }])

        self.app.data.insert('archive_versions', [{
            '_id_document': ids[0],
            'type': 'picture',
            'renditions': renditions,
        }])

        with patch.object(self.app.media, 'delete') as media_delete:
            get_resource_service('archive').delete_by_article_ids(ids)
            for key, rendition in renditions.items():
                media_delete.assert_any_call(rendition['media'])

    def _get_original(self, _id):
        return self.app.data.find_one(ARCHIVE, None, _id=_id)


class ArchiveTestCase(TestCase):
    def setUp(self):
        super().setUp()
        search_provider = {'_id': 1, 'source': 'ABC', 'name': 'ABC', 'search_provider': 'ABC'}
        self.app.data.insert('search_providers', [search_provider])
        if not registered_search_providers.get('ABC'):
            register_search_provider('ABC', fetch_endpoint='ABC', label='ABC')

    def test_validate_schedule(self):
        validate_schedule(utcnow() + timedelta(hours=2))

    def test_validate_schedule_at_utc_zero_hours(self):
        validate_schedule((utcnow() + timedelta(days=1)).replace(hour=0, minute=0, second=0, microsecond=0))

    def test_validate_schedule_date_with_datetime_as_string_raises_superdeskApiError(self):
        self.assertRaises(SuperdeskApiError, validate_schedule, "2015-04-27T10:53:48+00:00")

    def test_validate_schedule_date_with_datetime_in_past_raises_superdeskApiError(self):
        self.assertRaises(SuperdeskApiError, validate_schedule, utcnow() + timedelta(hours=-2))

    def _get_located_and_current_utc_ts(self):
        current_ts = utcnow()
        located = {"dateline": "city", "city_code": "Sydney", "state": "NSW", "city": "Sydney", "state_code": "NSW",
                   "country_code": "AU", "tz": "Australia/Sydney", "country": "Australia"}

        current_timestamp = datetime.fromtimestamp(current_ts.timestamp(), tz=timezone(located['tz']))
        if current_timestamp.month == 9:
            formatted_date = 'Sept {}'.format(current_timestamp.strftime('%-d'))
        elif 3 <= current_timestamp.month <= 7:
            formatted_date = current_timestamp.strftime('%B %-d')
        else:
            formatted_date = current_timestamp.strftime('%b %-d')

        return located, formatted_date, current_ts

    def test_format_dateline_to_format_when_only_city_is_present(self):
        located, formatted_date, current_ts = self._get_located_and_current_utc_ts()
        formatted_dateline = format_dateline_to_locmmmddsrc(located, current_ts)
        self.assertEqual(formatted_dateline, 'SYDNEY, %s %s -' % (formatted_date, get_default_source()))

    def test_format_dateline_to_format_when_only_city_and_state_are_present(self):
        located, formatted_date, current_ts = self._get_located_and_current_utc_ts()

        located['dateline'] = "city,state"
        formatted_dateline = format_dateline_to_locmmmddsrc(located, current_ts)
        self.assertEqual(formatted_dateline, 'SYDNEY, NSW, %s %s -' % (formatted_date, get_default_source()))

    def test_format_dateline_to_format_when_only_city_and_country_are_present(self):
        located, formatted_date, current_ts = self._get_located_and_current_utc_ts()

        located['dateline'] = "city,country"
        formatted_dateline = format_dateline_to_locmmmddsrc(located, current_ts)
        self.assertEqual(formatted_dateline, 'SYDNEY, AU, %s %s -' % (formatted_date, get_default_source()))

    def test_format_dateline_to_format_when_city_state_and_country_are_present(self):
        located, formatted_date, current_ts = self._get_located_and_current_utc_ts()

        located['dateline'] = "city,state,country"
        formatted_dateline = format_dateline_to_locmmmddsrc(located, current_ts)
        self.assertEqual(formatted_dateline, 'SYDNEY, NSW, AU, %s %s -' % (formatted_date, get_default_source()))

    def test_if_task_attributes_converted_to_objectid(self):
        doc = {
            'task': {
                'user': '562435231d41c835d7b5fb55',
                'desk': ObjectId("562435241d41c835d7b5fb5d"),
                'stage': 'test',
                'last_authoring_desk': 3245,
                'last_production_desk': None
            }
        }

        convert_task_attributes_to_objectId(doc)
        self.assertIsInstance(doc['task']['user'], ObjectId)
        self.assertEqual(doc['task']['desk'], ObjectId("562435241d41c835d7b5fb5d"))
        self.assertEqual(doc['task']['stage'], 'test')
        self.assertEqual(doc['task']['last_authoring_desk'], 3245)
        self.assertIsNone(doc['task']['last_production_desk'])

    def test_if_metadata_are_transtyped(self):
        """Check that date metadata in extra are transtyped correctly"""
        content_type = {
            "_id": "type_1",
            "label": "test_type",
            "editor": {"test_date_field": {"order": 1, "section": "header"}},
            "schema": {
                "test_date_field": {
                    "type": "date",
                    "required": False,
                    "enabled": True,
                    "nullable": True,
                }
            },
        }
        self.app.data.insert('content_types', [content_type])
        doc = {
            "_id": "transtype_1",
            "profile": "type_1",
            "extra": {"test_date_field": "2019-11-06T00:00:00+0000"},
        }

        transtype_metadata(doc)
        self.assertIsInstance(doc['extra']['test_date_field'], datetime)

    def test_if_no_source_defined_on_desk(self):
        desk = {'name': 'sports'}
        self.app.data.insert('desks', [desk])
        located, formatted_date, current_ts = self._get_located_and_current_utc_ts()
        doc = {
            '_id': '123',
            'task': {
                'desk': desk['_id'],
                'stage': desk['working_stage']
            },
            'dateline': {
                'located': located,
                'date': current_ts
            }
        }

        set_default_source(doc)
        self.assertEqual(doc['source'], get_default_source())
        self.assertEqual(doc['dateline']['source'], get_default_source())
        self.assertEqual(doc['dateline']['text'], 'SYDNEY, %s %s -' % (formatted_date, get_default_source()))

    def test_if_source_defined_on_desk(self):
        source = 'FOO'
        desk = {'name': 'sports', 'source': source}
        self.app.data.insert('desks', [desk])
        located, formatted_date, current_ts = self._get_located_and_current_utc_ts()
        doc = {
            '_id': '123',
            'task': {
                'desk': desk['_id'],
                'stage': desk['working_stage']
            },
            'dateline': {
                'located': located,
                'date': current_ts
            }
        }

        set_default_source(doc)
        self.assertEqual(doc['source'], source)
        self.assertEqual(doc['dateline']['source'], source)
        self.assertEqual(doc['dateline']['text'], 'SYDNEY, %s %s -' % (formatted_date, source))

    def test_if_ingest_provider_source_is_preserved(self):
        desk = {'name': 'sports', 'source': 'FOO'}
        self.app.data.insert('desks', [desk])
        ingest_provider = {'_id': 1, 'source': 'ABC'}
        self.app.data.insert('ingest_providers', [ingest_provider])
        located, formatted_date, current_ts = self._get_located_and_current_utc_ts()
        doc = {
            '_id': '123',
            'task': {
                'desk': desk['_id'],
                'stage': desk['working_stage']
            },
            'dateline': {
                'located': located,
                'date': current_ts
            },
            'ingest_provider': 1
        }

        set_default_source(doc)
        self.assertEqual(doc['source'], 'ABC')
        self.assertEqual(doc['dateline']['source'], 'ABC')
        self.assertEqual(doc['dateline']['text'], 'SYDNEY, %s %s -' % (formatted_date, 'ABC'))

    def test_if_ingest_provider_source_is_not_preserved_for_default_ingest(self):
        desk = {'name': 'sports', 'source': 'FOO'}
        self.app.data.insert('desks', [desk])
        ingest_provider = {'_id': 1, 'source': 'AAP'}
        self.app.data.insert('ingest_providers', [ingest_provider])
        located, formatted_date, current_ts = self._get_located_and_current_utc_ts()
        doc = {
            '_id': '123',
            'task': {
                'desk': desk['_id'],
                'stage': desk['working_stage']
            },
            'dateline': {
                'located': located,
                'date': current_ts
            },
            'ingest_provider': 1
        }

        set_default_source(doc)
        self.assertEqual(doc['source'], 'FOO')
        self.assertEqual(doc['dateline']['source'], 'FOO')
        self.assertEqual(doc['dateline']['text'], 'SYDNEY, %s %s -' % (formatted_date, 'FOO'))

    def test_if_search_provider_source_is_preserved(self):
        desk = {'name': 'sports', 'source': 'FOO'}
        self.app.data.insert('desks', [desk])
        doc = {
            '_id': '123',
            'task': {
                'desk': desk['_id'],
                'stage': desk['working_stage']
            },
            'type': 'picture',
            'ingest_provider': 1
        }

        set_default_source(doc)
        self.assertEqual(doc['source'], 'ABC')

    def test_if_item_has_source_then_search_provider_source_is_not_used(self):
        desk = {'name': 'sports', 'source': 'FOO'}
        self.app.data.insert('desks', [desk])
        doc = {
            '_id': '123',
            'task': {
                'desk': desk['_id'],
                'stage': desk['working_stage']
            },
            'type': 'picture',
            'ingest_provider': 1,
            'source': 'bar'
        }

        set_default_source(doc)
        self.assertEqual(doc['source'], 'bar')

    def test_if_image_caption_is_updated(self):
        body = """
        "body_html" : "<p>test 33</p>\n<!-- EMBED START Image {id: \"embedded9127149191\"} -->\n
        <figure><img src=\"http://localhost:5000/api/upload/58ff025eb611402decdb82e1/raw?_schema=http\" alt=\"aa\" />
        <figcaption>[--description--]</figcaption></figure>\n
        <!-- EMBED END Image {id: \"embedded9127149191\"} -->\n<p>faffaf</p>
        """
        changed_body = """
        "body_html" : "<p>test 33</p>\n<!-- EMBED START Image {id: \"embedded9127149191\"} -->\n
        <figure><img src=\"http://localhost:5000/api/upload/58ff025eb611402decdb82e1/raw?_schema=http\" alt=\"aa\" />
        <figcaption>new caption</figcaption></figure>\n
        <!-- EMBED END Image {id: \"embedded9127149191\"} -->\n<p>faffaf</p>
        """
        body = update_image_caption(body, 'embedded9127149191', 'new caption')
        self.assertEqual(body, changed_body)

    def test_update_associations(self):
        doc = {
            'fields_meta': {
                'body_html': {
                    'draftjsState': [{
                        'entityMap': {
                            '1': {
                                'mutability': 'MUTABLE',
                                'type': 'MEDIA',
                                'data': {
                                    'media': {
                                        'guid': 'guid1',
                                        'type': 'picture',
                                        'alt_text': 'media 1'
                                    }
                                }
                            },
                            '0': {
                                'mutability': 'MUTABLE',
                                'type': 'MEDIA',
                                'data': {
                                    'media': {
                                        'guid': 'guid0',
                                        'type': 'picture',
                                        'alt_text': 'media 0'
                                    }
                                }
                            },
                            '2': {
                                'mutability': 'MUTABLE',
                                'type': 'MEDIA',
                                'data': {
                                    'media': {
                                        'guid': 'guid2',
                                        'type': 'picture',
                                        'alt_text': 'media 2'
                                    }
                                }
                            }
                        }
                    }]}},
            'associations': {
                'editor_1': {'guid': 'guid10', 'type': 'picture10', 'alt_text': 'media 10'},
                'editor_7': {'guid': 'guid7', 'type': 'picture7', 'alt_text': 'media 7'},
                'featuremedia': {'guid': 'guid11', 'type': 'picture11'}}
        }

        update_associations(doc)

        self.assertEqual(len(doc['associations']), 5)
        self.assertEqual(doc['associations']['editor_0'], {'guid': 'guid0', 'type': 'picture', 'alt_text': 'media 0'})
        self.assertEqual(doc['associations']['editor_1'], {'guid': 'guid1', 'type': 'picture', 'alt_text': 'media 1'})
        self.assertEqual(doc['associations']['editor_2'], {'guid': 'guid2', 'type': 'picture', 'alt_text': 'media 2'})
        self.assertEqual(doc['associations']['editor_7'], None)
        self.assertEqual(doc['associations']['featuremedia'], {'guid': 'guid11', 'type': 'picture11'})

    def test_get_dateline_city_None(self):
        self.assertEqual(get_dateline_city(None), '')

    def test_get_dateline_city_located_as_none(self):
        self.assertEqual(get_dateline_city({'located': None}), '')

    def test_get_dateline_city_located_as_none_text_as_none(self):
        self.assertEqual(get_dateline_city({'located': None, 'text': None}), '')

    def test_get_dateline_city_from_text(self):
        self.assertEqual(get_dateline_city({'located': None, 'text': 'Sydney, 9 July AAP'}), 'Sydney')

    def test_get_dateline_city_from_located(self):
        self.assertEqual(get_dateline_city({'located': {'city': 'Melbourne'}, 'text': 'Sydney, 9 July AAP'}),
                         'Melbourne')

    def test_get_dateline_city_from_text_no_city(self):
        self.assertEqual(get_dateline_city({'located': {'city': None}, 'text': 'Sydney, 9 July AAP'}),
                         'Sydney')

    def test_get_dateline_city_from_located_with_country(self):
        self.assertEqual(get_dateline_city({'located': {'country': 'Canada'}, 'text': 'Sydney, 9 July AAP'}),
                         'Sydney')

    def test_get_dateline_city_from_text_with_city_state(self):
        self.assertEqual(get_dateline_city({'located': None, 'text': 'City, State, 9 July AAP'}), 'City, State')

    def test_firstpublished(self):
        """Check that "firstpublihed" field is set correctly

        the test create a story, check firstpublished field, then correct it
        and check again that correction is done and firstpublished has not changed
        """
        archive_service = superdesk.get_resource_service('archive')
        correct_service = superdesk.get_resource_service('archive_correct')
        publish_service = superdesk.get_resource_service('archive_publish')
        item = {'_id': 'foo',
                'guid': 'foo',
                'unique_name': 'foo',
                'type': 'text',
                'state': CONTENT_STATE.SUBMITTED,
                '_current_version': 1,
                'rewrite_of': 'bar'}
        archive_service.create([item])
        with mock.patch.object(publish, 'utcnow', lambda: NOW):
            publish_service.patch('foo', {'body_html': 'original'})
        created = publish_service.find_one(None, _id='foo')
        self.assertEqual(NOW, created['firstpublished'])
        correct_service.patch('foo', {'body_html': 'corrected'})
        # we try to update to check that "firstpublished" is not modified
        # note that utcnow MUST NOT be mocked here, else the test would be pointless
        corrected = publish_service.find_one(None, _id='foo')
        self.assertEqual('corrected', corrected['body_html'])
        self.assertEqual(NOW, corrected['firstpublished'])


class ArchiveCommonTestCase(unittest.TestCase):

    def test_broadcast_content(self):
        content = {
            'genre': [{'name': 'Broadcast Script', 'qcode': 'Broadcast Script'}]
        }

        self.assertTrue(is_genre(content, BROADCAST_GENRE))

    def test_broadcast_content_if_genre_is_none(self):
        content = {
            'genre': None
        }

        self.assertFalse(is_genre(content, BROADCAST_GENRE))

    def test_broadcast_content_if_genre_is_empty_list(self):
        content = {
            'genre': []
        }

        self.assertFalse(is_genre(content, BROADCAST_GENRE))

    def test_broadcast_content_if_genre_is_other_than_broadcast(self):
        content = {
            'genre': [{'name': 'Article', 'qcode': 'Article'}]
        }

        self.assertFalse(is_genre(content, BROADCAST_GENRE))
        self.assertTrue(is_genre(content, 'Article'))

    def test_get_utc_schedule(self):
        embargo_date = utcnow() + timedelta(minutes=10)
        content = {
            'embargo': embargo_date
        }
        utc_schedule = get_utc_schedule(content, 'embargo')
        self.assertEqual(utc_schedule, embargo_date)


class ExpiredArchiveContentTestCase(TestCase):

    def setUp(self):
        try:
            from apps.archive.commands import RemoveExpiredContent
        except ImportError:
            self.fail("Could not import class under test (RemoveExpiredContent).")
        else:
            self.class_under_test = RemoveExpiredContent
            self.published_items = [
                {
                    '_id': 'item1', 'item_id': 'item1', 'headline': 'headline',
                    'source': 'aap', 'body_html': 'test', '_current_version': 3,
                    'moved_to_legal': True
                },
                {
                    '_id': 'item2', 'item_id': 'item2', 'headline': 'headline',
                    'source': 'aap', 'body_html': 'test', '_current_version': 3,
                    'moved_to_legal': True
                },
                {
                    '_id': 'item3', 'item_id': 'item3', 'headline': 'headline',
                    'source': 'aap', 'body_html': 'test', '_current_version': 3,
                    'moved_to_legal': False
                },
                {
                    '_id': 'item4', 'item_id': 'item4', 'headline': 'headline',
                    'source': 'aap', 'body_html': 'test', '_current_version': 3,
                    'moved_to_legal': True
                }
            ]

            self.queue_items = [
                {
                    '_id': 'item1', 'item_id': 'item1', 'headline': 'headline',
                    'item_version': 3, 'moved_to_legal': True
                },
                {
                    '_id': 'item2', 'item_id': 'item2', 'headline': 'headline',
                    'source': 'aap', 'body_html': 'test', 'item_version': 3,
                    'moved_to_legal': True
                },
                {
                    '_id': 'item3', 'item_id': 'item3', 'headline': 'headline',
                    'source': 'aap', 'body_html': 'test', 'item_version': 3,
                    'moved_to_legal': False
                },
                {
                    '_id': 'item4', 'item_id': 'item4', 'headline': 'headline',
                    'source': 'aap', 'body_html': 'test', 'item_version': 3,
                    'moved_to_legal': False
                },
                {
                    '_id': 'item5', 'item_id': 'item4', 'headline': 'headline',
                    'source': 'aap', 'body_html': 'test', 'item_version': 3,
                    'moved_to_legal': True
                }
            ]

            self.app.data.insert('published', self.published_items)
            self.app.data.insert('publish_queue', self.queue_items)

    def test_items_moved_to_legal_success(self):
        test_items = dict()
        test_items['item1'] = self.published_items[0]
        test_items['item2'] = self.published_items[1]
        result = self.class_under_test().check_if_items_imported_to_legal_archive(test_items)
        self.assertDictEqual(result, {})

    def test_items_moved_to_legal_fail_if_published_item_not_moved(self):
        test_items = dict()
        test_items['item2'] = self.published_items[1]
        test_items['item3'] = self.published_items[2]
        result = self.class_under_test().check_if_items_imported_to_legal_archive(test_items)
        self.assertIn('item3', result)

    def test_items_moved_to_legal_fail_if_published_queue_item_not_moved(self):
        test_items = dict()
        test_items['item2'] = self.published_items[1]
        test_items['item3'] = self.published_items[3]
        result = self.class_under_test().check_if_items_imported_to_legal_archive(test_items)
        self.assertIn('item3', result)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging

from superdesk.resource import Resource
from superdesk.services import BaseService
from superdesk.errors import SuperdeskApiError
from flask_babel import _

logger = logging.getLogger(__name__)


class RuleSetsResource(Resource):
    schema = {
        'name': {
            'type': 'string',
            'iunique': True,
            'required': True,
            'nullable': False,
            'empty': False,
            'minlength': 1
        },
        'rules': {
            'type': 'list'
        }
    }

    privileges = {'POST': 'rule_sets', 'DELETE': 'rule_sets', 'PATCH': 'rule_sets'}


class RuleSetsService(BaseService):

    def update(self, id, updates, original):
        """
        Overriding to set the value of "new" attribute of rules to empty string if it's None.
        """

        for rule in updates.get('rules', {}):
            if rule['new'] is None:
                rule['new'] = ''

        return super().update(id, updates, original)

    def on_delete(self, doc):
        if self.backend.find_one('ingest_providers', req=None, rule_set=doc['_id']):
            raise SuperdeskApiError.forbiddenError(_("Cannot delete Rule set as it's associated with channel(s)."))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import logging
import json
from eve.utils import ParsedRequest
from eve.versioning import resolve_document_version
from flask import request
from apps.archive.common import CUSTOM_HATEOAS, insert_into_versions, get_user, \
    ITEM_CREATE, BROADCAST_GENRE, is_genre
from apps.packages import PackageService
from superdesk.metadata.packages import GROUPS
from superdesk.resource import Resource, build_custom_hateoas
from superdesk.services import BaseService
from superdesk.metadata.utils import item_url
from superdesk.metadata.item import CONTENT_TYPE, CONTENT_STATE, ITEM_TYPE, ITEM_STATE, PUBLISH_STATES
from superdesk import get_resource_service, config
from superdesk.errors import SuperdeskApiError
from apps.archive.archive import SOURCE
from apps.publish.content.common import ITEM_CORRECT, ITEM_PUBLISH
from superdesk.utc import utcnow
from flask_babel import _


logger = logging.getLogger(__name__)
# field to be copied from item to broadcast item
FIELDS_TO_COPY = ['urgency', 'priority', 'anpa_category', 'type', 'profile',
                  'subject', 'dateline', 'slugline', 'place']
ARCHIVE_BROADCAST_NAME = 'archive_broadcast'


class ArchiveBroadcastResource(Resource):
    endpoint_name = ARCHIVE_BROADCAST_NAME
    resource_title = endpoint_name

    url = 'archive/<{0}:item_id>/broadcast'.format(item_url)
    schema = {
        'desk': Resource.rel('desks', embeddable=False, required=False, nullable=True)
    }
    resource_methods = ['POST']
    item_methods = []
    privileges = {'POST': ARCHIVE_BROADCAST_NAME}


class ArchiveBroadcastService(BaseService):

    packageService = PackageService()

    def create(self, docs):
        service = get_resource_service(SOURCE)
        item_id = request.view_args['item_id']
        item = service.find_one(req=None, _id=item_id)
        doc = docs[0]

        self._valid_broadcast_item(item)

        desk_id = doc.get('desk')
        desk = None

        if desk_id:
            desk = get_resource_service('desks').find_one(req=None, _id=desk_id)

        doc.pop('desk', None)
        doc['task'] = {}
        if desk:
            doc['task']['desk'] = desk.get(config.ID_FIELD)
            doc['task']['stage'] = desk.get('working_stage')

        doc['task']['user'] = get_user().get('_id')
        genre_list = get_resource_service('vocabularies').find_one(req=None, _id='genre') or {}
        broadcast_genre = [{'qcode': genre.get('qcode'), 'name': genre.get('name')}
                           for genre in genre_list.get('items', [])
                           if genre.get('qcode') == BROADCAST_GENRE and genre.get('is_active')]

        if not broadcast_genre:
            raise SuperdeskApiError.badRequestError(
                message=_("Cannot find the {genre} genre.").format(genre=BROADCAST_GENRE))

        doc['broadcast'] = {
            'status': '',
            'master_id': item_id,
            'rewrite_id': item.get('rewritten_by')
        }

        doc['genre'] = broadcast_genre
        doc['family_id'] = item.get('family_id')

        for key in FIELDS_TO_COPY:
            doc[key] = item.get(key)

        resolve_document_version(document=doc, resource=SOURCE, method='POST')
        service.post(docs)
        insert_into_versions(id_=doc[config.ID_FIELD])
        build_custom_hateoas(CUSTOM_HATEOAS, doc)
        return [doc[config.ID_FIELD]]

    def _valid_broadcast_item(self, item):
        """Validates item for broadcast.

        Broadcast item can only be created for Text or Pre-formatted item.
        Item state needs to be Published or Corrected

        :param dict item: Item from which the broadcast item will be created
        """
        if not item:
            raise SuperdeskApiError.notFoundError(
                message=_("Cannot find the requested item id."))

        if not item.get(ITEM_TYPE) in [CONTENT_TYPE.TEXT, CONTENT_TYPE.PREFORMATTED]:
            raise SuperdeskApiError.badRequestError(message=_("Invalid content type."))

        if item.get(ITEM_STATE) not in [CONTENT_STATE.CORRECTED, CONTENT_STATE.PUBLISHED]:
            raise SuperdeskApiError.badRequestError(message=_("Invalid content state."))

    def _get_broadcast_items(self, ids, include_archived_repo=False):
        """Returns list of broadcast items.

        Get the broadcast items for the master_id

        :param list ids: list of item ids
        :param include_archived_repo True if archived repo needs to be included in search, default is False
        :return list: list of broadcast items
        """
        query = {
            'query': {
                'filtered': {
                    'filter': {
                        'bool': {
                            'must': {'term': {'genre.name': BROADCAST_GENRE}},
                            'should': {'terms': {'broadcast.master_id': ids}}
                        }
                    }
                }
            }
        }

        req = ParsedRequest()
        repos = 'archive,published'
        if include_archived_repo:
            repos = 'archive,published,archived'

        req.args = {'source': json.dumps(query), 'repo': repos}
        return get_resource_service('search').get(req=req, lookup=None)

    def get_broadcast_items_from_master_story(self, item, include_archived_repo=False):
        """Get the broadcast items from the master story.

        :param dict item: master story item
        :param include_archived_repo True if archived repo needs to be included in search, default is False
        :return list: returns list of broadcast items
        """
        if is_genre(item, BROADCAST_GENRE):
            return []

        ids = [str(item.get(config.ID_FIELD))]
        return list(self._get_broadcast_items(ids, include_archived_repo))

    def on_broadcast_master_updated(self, item_event, item, rewrite_id=None):
        """Runs when master item is updated.

        This event is called when the master story is corrected, published, re-written

        :param str item_event: Item operations
        :param dict item: item on which operation performed.
        :param str rewrite_id: re-written story id.
        """
        status = ''

        if not item or is_genre(item, BROADCAST_GENRE):
            return

        elif item_event == ITEM_CREATE and rewrite_id:
            status = 'Master Story Re-written'
        elif item_event == ITEM_PUBLISH:
            status = 'Master Story Published'
        elif item_event == ITEM_CORRECT:
            status = 'Master Story Corrected'

        broadcast_items = self.get_broadcast_items_from_master_story(item)

        if not broadcast_items:
            return

        processed_ids = set()
        for broadcast_item in broadcast_items:
            try:
                if broadcast_item.get('lock_user'):
                    continue

                updates = {
                    'broadcast': broadcast_item.get('broadcast'),
                }

                if status:
                    updates['broadcast']['status'] = status

                if not updates['broadcast']['rewrite_id'] and rewrite_id:
                    updates['broadcast']['rewrite_id'] = rewrite_id

                if not broadcast_item.get(config.ID_FIELD) in processed_ids:
                    self._update_broadcast_status(broadcast_item, updates)
                    # list of ids that are processed.
                    processed_ids.add(broadcast_item.get(config.ID_FIELD))
            except Exception:
                logger.exception('Failed to update status for the broadcast item {}'.
                                 format(broadcast_item.get(config.ID_FIELD)))

    def _update_broadcast_status(self, item, updates):
        """Update the status of the broadcast item.

        :param dict item: broadcast item to be updated
        :param dict updates: broadcast updates
        """
        # update the published collection as well as archive.
        if item.get(ITEM_STATE) in \
                {CONTENT_STATE.PUBLISHED, CONTENT_STATE.CORRECTED, CONTENT_STATE.KILLED, CONTENT_STATE.RECALLED}:
            get_resource_service('published').update_published_items(item.get(config.ID_FIELD),
                                                                     'broadcast', updates.get('broadcast'))

        archive_item = get_resource_service(SOURCE).find_one(req=None, _id=item.get(config.ID_FIELD))
        get_resource_service(SOURCE).system_update(archive_item.get(config.ID_FIELD), updates, archive_item)

    def remove_rewrite_refs(self, item):
        """Remove the rewrite references from the broadcast item if the re-write is spiked.

        :param dict item: Re-written article of the original story
        """
        if is_genre(item, BROADCAST_GENRE):
            return

        query = {
            'query': {
                'filtered': {
                    'filter': {
                        'and': [
                            {'term': {'genre.name': BROADCAST_GENRE}},
                            {'term': {'broadcast.rewrite_id': item.get(config.ID_FIELD)}}
                        ]
                    }
                }
            }
        }

        req = ParsedRequest()
        req.args = {'source': json.dumps(query)}
        broadcast_items = list(get_resource_service(SOURCE).get(req=req, lookup=None))

        for broadcast_item in broadcast_items:
            try:
                updates = {
                    'broadcast': broadcast_item.get('broadcast', {})
                }

                updates['broadcast']['rewrite_id'] = None

                if 'Re-written' in updates['broadcast']['status']:
                    updates['broadcast']['status'] = ''

                self._update_broadcast_status(broadcast_item, updates)
            except Exception:
                logger.exception('Failed to remove rewrite id for the broadcast item {}'.
                                 format(broadcast_item.get(config.ID_FIELD)))

    def reset_broadcast_status(self, updates, original):
        """Reset the broadcast status if the broadcast item is updated.

        :param dict updates: updates to the original document
        :param dict original: original document
        """
        if original.get('broadcast') and original.get('broadcast').get('status', ''):
            broadcast_updates = {
                'broadcast': original.get('broadcast'),
            }

            broadcast_updates['broadcast']['status'] = ''
            self._update_broadcast_status(original, broadcast_updates)
            updates.update(broadcast_updates)

    def spike_item(self, original):
        """If Original item is re-write then it will remove the reference from the broadcast item.

        :param: dict original: original document
        """
        broadcast_items = [item for item in self.get_broadcast_items_from_master_story(original)
                           if item.get(ITEM_STATE) not in PUBLISH_STATES]
        spike_service = get_resource_service('archive_spike')

        for item in broadcast_items:
            id_ = item.get(config.ID_FIELD)
            try:
                self.packageService.remove_spiked_refs_from_package(id_)
                updates = {ITEM_STATE: CONTENT_STATE.SPIKED}
                resolve_document_version(updates, SOURCE, 'PATCH', item)
                spike_service.patch(id_, updates)
                insert_into_versions(id_=id_)
            except Exception:
                logger.exception(message="Failed to spike the related broadcast item {}.".format(id_))

        if original.get('rewrite_of') and original.get(ITEM_STATE) not in PUBLISH_STATES:
            self.remove_rewrite_refs(original)

    def kill_broadcast(self, updates, original, operation):
        """Kill the broadcast items

        :param dict updates: Updates to the item
        :param dict original: original item
        :param str operation: Kill or Takedown operation
        :return:
        """
        broadcast_items = [item for item in self.get_broadcast_items_from_master_story(original)
                           if item.get(ITEM_STATE) in PUBLISH_STATES]

        correct_service = get_resource_service('archive_correct')
        kill_service = get_resource_service('archive_{}'.format(operation))

        for item in broadcast_items:
            item_id = item.get(config.ID_FIELD)
            packages = self.packageService.get_packages(item_id)

            processed_packages = set()
            for package in packages:
                if str(package[config.ID_FIELD]) in processed_packages or \
                        package.get(ITEM_STATE) == CONTENT_STATE.RECALLED:
                    continue
                try:
                    if package.get(ITEM_STATE) in {CONTENT_STATE.PUBLISHED, CONTENT_STATE.CORRECTED}:
                        package_updates = {
                            config.LAST_UPDATED: utcnow(),
                            GROUPS: self.packageService.remove_group_ref(package, item_id)
                        }

                        refs = self.packageService.get_residrefs(package_updates)
                        if refs:
                            correct_service.patch(package.get(config.ID_FIELD), package_updates)
                        else:
                            package_updates['body_html'] = updates.get('body_html', '')
                            kill_service.patch(package.get(config.ID_FIELD), package_updates)

                        processed_packages.add(package.get(config.ID_FIELD))
                    else:
                        package_list = self.packageService.remove_refs_in_package(package,
                                                                                  item_id, processed_packages)

                        processed_packages = processed_packages.union(set(package_list))
                except Exception:
                    logger.exception('Failed to remove the broadcast item {} from package {}'.format(
                        item_id, package.get(config.ID_FIELD)
                    ))

            kill_service.kill_item(updates, item)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import logging
import json
from superdesk.io.registry import register_feeding_service, register_feeding_service_parser
from superdesk.io.feeding_services.http_base_service import HTTPFeedingServiceBase
from superdesk.errors import IngestApiError, SuperdeskIngestError
import requests
import superdesk
from superdesk.io.feed_parsers import nitf
from lxml import etree
from superdesk.utc import utcnow
from datetime import timedelta, datetime

logger = logging.getLogger(__name__)


class APMediaFeedingService(HTTPFeedingServiceBase):
    """
    Feeding Service class which can retrieve articles from Associated Press Media API
    """

    NAME = 'ap media api'

    label = 'AP Media API'

    fields = [
        {
            'id': 'api_url', 'type': 'text', 'label': 'AP Media API URL',
            'required': True, 'default_value': 'https://api.ap.org/media/v/content/feed'
        },
        {
            'id': 'products_url', 'type': 'text', 'label': 'AP Media API Products URL',
            'required': True, 'default_value': 'https://api.ap.org/media/v/account/plans'
        },
        {
            'id': 'apikey', 'type': 'text', 'label': 'API Key',
            'placeholder': 'API key for access to the API', 'required': True
        },
        {
            'id': 'productList', 'type': 'text', 'label': 'Product List',
            'placeholder': 'Use coma separated product id''s for multiple products, empty for all ', 'required': False
        },
        {
            'id': 'availableProducts', 'type': 'text', 'label': 'All Available Products',
            'readonly': True
        },
        {
            'id': 'next_link', 'type': 'text', 'label': 'Next Link',
            'readonly': True
        },
        {
            'id': 'recoverytime', 'type': 'text', 'label': 'Number of hours to recover', 'default_value': '',
            'placeholder': 'Specifying a number will restart the feed from that number of hours in the past'
        }
    ]

    HTTP_AUTH = False

    HTTP_TIMEOUT = 40.0

    def config_test(self, provider=None):
        self._get_products(provider)
        original = superdesk.get_resource_service('ingest_providers').find_one(req=None, _id=provider.get('_id'))
        # If there has been a change in the required products then reset the next link
        if original and (original.get('config', {}).get('productList', '') != provider.get('config', {}).get(
                'productList', '') or original.get('config', {}).get('recoverytime', '') !=
                provider.get('config', {}).get('recoverytime', '')):
            provider['config']['next_link'] = None

    def _get_products(self, provider):
        """
        Get the products that are available for the API Key, effectively ensuring that the key is valid and provide an
         indication of the product codes available in the UI.
        :param provider:
        :return:
        """
        api_key = provider.get('config', {}).get('apikey')
        r = requests.get(provider.get('config', {}).get('products_url') + '?apikey={}'.format(api_key),
                         timeout=self.HTTP_TIMEOUT, verify=False, allow_redirects=True)
        r.raise_for_status()
        productList = []
        products = json.loads(r.text)
        for plan in products.get('data', {}).get('plans'):
            for entitlement in plan.get('entitlements'):
                productList.append('{}'.format(entitlement.get('id')))
        provider['config']['availableProducts'] = ','.join(productList)

    def prepare_href(self, href, mimetype=None):
        href = href + '&apikey=' + self.provider.get('config', {}).get('apikey') if '?' in href else \
            href + '?apikey=' + self.provider.get('config', {}).get('apikey')
        return href

    def _update(self, provider, update):
        self.HTTP_URL = provider.get('config', {}).get('api_url', '')
        self.provider = provider

        # Set the apikey parameter we're going to use it on all calls
        params = dict()
        params['apikey'] = provider.get('config', {}).get('apikey')

        # Use the next link if one is available in the config
        if provider.get('config', {}).get('next_link'):
            r = self.get_url(url=provider.get('config', {}).get('next_link'), params=params,
                             verify=False, allow_redirects=True)
            r.raise_for_status()
        else:
            id_list = provider.get('config', {}).get('productList', '').strip()
            recovery_time = provider.get('config', {}).get('recoverytime', '1')
            recovery_time = recovery_time.strip() if recovery_time else ''
            if recovery_time == '':
                recovery_time = '1'
            start = datetime.strftime(utcnow() - timedelta(hours=int(recovery_time)), '%Y-%m-%dT%H:%M:%SZ')
            # If there has been a list of products defined then we format them for the request, if not all
            # allowed products will be returned.
            if id_list:
                # we remove spaces and empty values from id_list to do a clean list
                id_list = ' OR '.join([id_.strip() for id_ in id_list.split(',') if id_.strip()])
                params['q'] = 'productid:(' + id_list + ') AND mindate:>{}'.format(start)
            else:
                params['q'] = 'mindate:>{}'.format(start)
            params['page_size'] = '100'
            params['versions'] = 'all'

            logger.info('AP Media Start/Recovery time: {} params {}'.format(recovery_time, params))
            r = self.get_url(params=params, verify=False, allow_redirects=True)
            r.raise_for_status()
        try:
            response = json.loads(r.text)
        except Exception:
            raise IngestApiError.apiRequestError(Exception('error parsing response'))

        nextLink = response.get('data', {}).get('next_page')
        # Got the same next link as last time so nothing new
        if nextLink == provider.get('config', {}).get('next_link'):
            logger.info('Nothing new from AP Media')
            return []

        parser = self.get_feed_parser(provider)
        parsed_items = []
        for item in response.get('data', {}).get('items', []):
            try:
                # Get the item meta data
                r = self.get_url(url=item.get('item', {}).get('uri'),
                                 params={'apikey': provider.get('config', {}).get('apikey')}, verify=False,
                                 allow_redirects=True)
                logger.info('Get AP meta data for "{}" uri: {}'.format(item.get('item', {}).get('headline'),
                                                                       item.get('item', {}).get('uri')))
                r.raise_for_status()
                complete_item = json.loads(r.text)

                # Get the nitf rendition of the item
                nitf_ref = complete_item.get('data', {}).get('item', {}).get('renditions', {}).get('nitf', {}).get(
                    'href')
                if nitf_ref:
                    logger.info('Get AP nitf : {}'.format(nitf_ref))
                    r = self.get_url(url=nitf_ref, params={'apikey': provider.get('config', {}).get('apikey')},
                                     verify=False, allow_redirects=True)
                    r.raise_for_status()
                    root_elt = etree.fromstring(r.content)
                    nitf_item = nitf.NITFFeedParser().parse(root_elt)
                    complete_item['nitf'] = nitf_item
                else:
                    if item.get('item', {}).get('type') == 'text':
                        logger.warning('No NITF for story {}'.format(item.get('item', {}).get('uri')))

                parsed_items.append(parser.parse(complete_item, provider))

            # Any exception processing an indivisual item is swallowed
            except Exception as ex:
                logger.exception(ex)

        # Save the link for next time
        upd_provider = provider.get('config')
        upd_provider['next_link'] = nextLink
        upd_provider['recoverytime'] = None
        update['config'] = upd_provider

        return [parsed_items]


register_feeding_service(APMediaFeedingService)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import re
from superdesk.text_utils import get_text

p = re.compile('(?i)(?<=[.?!])\\S+(?=[a-z])')


def populate(item, **kwargs):
    """Populate the abstract field with the first sentence of the body"""

    # get the list of sentences of the body
    if not item.get('body_html', None):
        item['abstract'] = 'No body found to use for abstract...'
    else:
        sentences = p.split(item['body_html'])

        # chop the first sentence to size for abstract (64)
        if sentences and len(sentences) > 0:
            item['abstract'] = get_text(sentences[0][:64]).strip()

    return item


name = 'populate_abstract'
label = 'Populate Abstract'
order = 3
shortcut = 'a'
callback = populate
desks = ['POLITICS']
access_type = 'frontend'
action_type = 'direct'
replace_type = 'keep-style-replace'



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013-2019 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
import os
import shutil
import subprocess
import tempfile
import json
from urllib.parse import urljoin
import requests
from os.path import abspath, expanduser
from superdesk.errors import SuperdeskApiError
from superdesk.text_checkers.spellcheckers import CAP_SPELLING, CAP_GRAMMAR
from superdesk.text_checkers.spellcheckers.base import SpellcheckerBase

logger = logging.getLogger(__name__)

PATH_CHECK = "gc_text/fr"
PATH_SUGGEST = "suggest/fr"
# We have an option to select CLI or the HTTP Server, because each options has pros/cons:
# - CLI expects the text to be saved in a file each time
# - Server can't handle spelling suggestion for now
# Grammalecte author has been contacted, and we'll see which options suits the best to
# Superdesk, and if we can contribute to add missing features.
OPT_URL = "GRAMMALECTE_URL"
OPT_CLI = "GRAMMALECTE_CLI"

#: dictionary
OPT_CONFIG = "GRAMMALECTE_CONFIG"

# debug flag to activate spelling suggestion when doing a whole text check
# this is resource consuming and getSuggestions should be used instead.
# This flag should be removed at some point. Note that it's only working with CLI
SPELLING_SUGGESTIONS = False


class Grammalecte(SpellcheckerBase):
    """Grammelecte grammar/spelling/style checker integration

    This class works with either the Grammalecte CLI or the Grammalecte server (both are
    downloadable at https://grammalecte.net).
    We use the CLI in priority because it handles spelling suggestions,
    which is not the case with the server.

    The GRAMMELECTE_CLI setting (or environment variable) can be set to
    ``grammelecte-cli.py`` path.

    The GRAMMALECTE_URL setting (or environment variable) can be set to the base URL of
    the server.
    e.g.: if you run Grammelect server with ``grammalecte-server.py -t -p 9999``, you'll
    have to put in the settings ``GRAMMALECTE_URL = http://localhost:9999`` (or use the
    environment variable of the same name).

    If GRAMMALECTE_CLI or GRAMMALECTE_URL are specified, they are used (if both are
    specified, only GRAMMALECTE_CLI will be used). If none is specified, the CLI is search
    in executables path, and if not found default URL (http://localhost:8080) will be used
    .

    Grammelecte behaviour can be specified using GRAMMALECTE_CONFIG setting, which must be
    a dictionary mapping grammalecte option names to their boolean value.
    Check ``grammalecte-cli.py -lo`` to get option names.
    """

    name = "grammalecte"
    capacities = (CAP_SPELLING, CAP_GRAMMAR)
    languages = ['fr']

    def __init__(self, app):
        super().__init__(app)

        self._grammalecte_config = None

        # we have two ways to use Grammalecte: CLI and server
        # we use CLI in priority because it handles suggestions for spelling.
        conf_cli_path = self.config.get(OPT_CLI, os.environ.get(OPT_CLI))

        if conf_cli_path is not None:
            # grammalecte_cli.py path is specified, we use CLI
            self.use_cli = True
            self._cli = abspath(expanduser(conf_cli_path))
            return

        conf_base_url = self.config.get(OPT_URL, os.environ.get(OPT_URL))
        if conf_base_url is not None:
            # grammalecte server URL is specified, we use server
            self.base_url = conf_base_url
            self.use_cli = False
            return

        # nothing specified, we try to find the grammalecte_cli.py path, else we use server with default URL

        self._cli = shutil.which("grammalecte-cli.py")
        if self._cli is None:
            # no CLI found, we'll use default URL
            self.use_cli = False
            self.base_url = "http://localhost:8080"
            return
        else:
            self.use_cli = True

    @property
    def grammalecte_config(self):
        """Retrieve Grammalecte config from settings.py or environment variables

        config is cached once retrieved
        """
        if self._grammalecte_config is None:
            config = self.config.get(OPT_CONFIG, {})
            try:
                env_config = json.loads(os.environ[OPT_CONFIG])
            except (KeyError, json.JSONDecodeError):
                env_config = {}
            if not isinstance(config, dict) or not isinstance(env_config, dict):
                logger.warning("Invalid type for Grammalecte configuration, must be a dictionary")
                self._grammalecte_config = {}
                return
            config.update(env_config)
            if not all({isinstance(v, bool) for v in config.values()}):
                logger.warning("Invalid values for Grammalecte configuration, boolean must be used")
                self._grammalecte_config = {}
                return
            self._grammalecte_config = config

        return self._grammalecte_config

    def grammalecte2superdesk(self, text, json_data):
        """Convert Grammalecte JSON to format used by Superdesk

        :param str text: original text being checked
        :param dict json_data: json return by Grammalecte
        :return dict: json used by Superdesk
        """
        corrections_data = json_data['data']
        err_list = []
        check_data = {'errors': err_list}

        for corr_data in corrections_data:
            paragraph_idx = corr_data.get('iParagraph', 1) - 1
            start_p_index = 0
            for idx in range(paragraph_idx):
                start_p_index = text.index('\n', start_p_index + 1)
            if paragraph_idx:
                # we must add the line feed character
                start_p_index += 1

            grammar_errors = corr_data.get("lGrammarErrors", [])
            spelling_errors = corr_data.get("lSpellingErrors", [])
            for errors, error_type in ((grammar_errors, 'grammar'),
                                       (spelling_errors, 'spelling')):
                for error in errors:
                    start = start_p_index + error['nStart']
                    end = start_p_index + error['nEnd']
                    ercorr_data = {
                        'startOffset': start,
                        'text': text[start:end],
                        'type': error_type,
                    }

                    if 'aSuggestions' in error:
                        ercorr_data['suggestions'] = [{'text': s} for s in error['aSuggestions']]

                    try:
                        ercorr_data['message'] = error['sMessage']
                    except KeyError:
                        pass

                    err_list.append(ercorr_data)

        return check_data

    def _check_cli(self, text):
        with tempfile.NamedTemporaryFile() as f:
            f.write(text.encode('utf-8'))
            f.flush()
            extra_args = []
            if SPELLING_SUGGESTIONS:
                extra_args.append('--with_spell_sugg')
            for opt, activate in self.grammalecte_config.items():
                if activate:
                    extra_args.extend(['-on', opt])
                else:
                    extra_args.extend(['-off', opt])
            cmp_proc = subprocess.run(['/usr/bin/env', 'python3', self._cli,
                                       '--file', f.name, '--json'] + extra_args,
                                      stdout=subprocess.PIPE,
                                      stderr=subprocess.PIPE,
                                      check=True)

            out = cmp_proc.stdout.decode('utf-8')
            return self.grammalecte2superdesk(text, json.loads(out))

    def _check_server(self, text):
        check_url = urljoin(self.base_url, PATH_CHECK)
        r = requests.post(check_url, data={"text": text,
                                           "options": json.dumps(self.grammalecte_config)}, timeout=self.CHECK_TIMEOUT)
        if r.status_code != 200:
            raise SuperdeskApiError.internalError("Unexpected return code from Grammalecte")
        return self.grammalecte2superdesk(text, r.json())

    def check(self, text, language=None):
        return self._check_cli(text) if self.use_cli else self._check_server(text)

    def _suggest_cli(self, text):
        cmp_proc = subprocess.run(['/usr/bin/env', 'python3', self._cli,
                                   '--suggest', text, '--json'],
                                  stdout=subprocess.PIPE,
                                  stderr=subprocess.PIPE,
                                  check=True)

        out = cmp_proc.stdout.decode('utf-8')
        suggestions = json.loads(out).get('aSuggestions', [])
        return {'suggestions': self.list2suggestions(suggestions)}

    def _suggest_server(self, text):
        if self.version_tuple < (1, 2):
            logger.warning("Suggestions not available with this server version")
            return {'suggestions': []}
        check_url = urljoin(self.base_url, PATH_SUGGEST)
        r = requests.post(check_url, data={"token": text}, timeout=self.SUGGEST_TIMEOUT)
        if r.status_code != 200:
            raise SuperdeskApiError.internalError("Unexpected return code from Grammalecte")

        suggestions = r.json().get('suggestions', [])
        return {'suggestions': self.list2suggestions(suggestions)}

    def suggest(self, text, language=None):
        return self._suggest_cli(text) if self.use_cli else self._suggest_server(text)

    def _available_cli(self):
        """Check if grammalecte-cli is available, and parse version"""
        if self._cli is None:
            logger.warning("can't find grammalecte-cli.py in path, can't use Grammalecte.")
            return False
        try:
            with tempfile.NamedTemporaryFile() as f:
                # we use empty file to get version, else grammalecte-cli will wait for input
                # (there is no --version option at the time of writing)
                cmp_proc = subprocess.run([self._cli, '--file', f.name],
                                          stdout=subprocess.PIPE,
                                          stderr=subprocess.PIPE,
                                          check=True)

                out = cmp_proc.stdout.decode('utf-8')
                version = out.strip().split()[1][1:]
        except Exception as e:
            logger.warning("can't use Grammalecte: {e}".format(e=e))
            return False

        logger.info("Grammalecte v{version} detected (CLI)".format(version=version))
        self.version = version
        return True

    def _available_server(self):
        """Check if grammalecte-server is launched at expected URL, and retrieve Grammalecte version"""
        check_url = urljoin(self.base_url, PATH_CHECK)
        try:
            r = requests.post(check_url, data={"text": ""}, timeout=self.CHECK_TIMEOUT)
        except Exception as e:
            logger.warning(
                "can't request Grammalecte URL ({check_url}): {e}".
                format(check_url=check_url, e=e))
            return False
        if r.status_code != 200:
            logger.warning(
                "Grammalecte URL ({check_url}) is not returning the expected status"
                .format(check_url=check_url))
            return False

        data = r.json()
        if data['program'] != 'grammalecte-fr':
            logger.warning("unexpected program: {program}".format(program=data['program']))
            return False
        version = data['version']
        logger.info("Grammalecte v{version} detected (server)".format(version=version))
        self.version_tuple = tuple(int(n) for n in version.split('.'))
        if self.version_tuple < (1, 2):
            logger.warning("Suggestions are only available with server version >= 1.2")
        self.version = version
        return True

    def available(self):
        try:
            return self._available_cli() if self.use_cli else self._available_server()
        except Exception as e:
            logger.warning("Can't check Grammalecte availability: {e}".format(e=e))
            return False



"""
Superdesk OAuth support

.. versionadded:: 1.9

There is ``oauth`` resource for authenticating users by email.
Email must be provided by trusted identity service, eg. Google.

The authentication flow with js client is:

- Client opens server api url in a new window,.
- There is a redirect to auth provider (eg. Google).
- Once authenticated user is redirected back to server api url.
- There user email from auth provider is used to authenticate user.
- Template is rendered with session data which sends message to parent window and popup is closed.

.. autofunction:: auth_user

"""

import superdesk

from flask import render_template
from apps.auth.auth import AuthResource
from apps.auth.service import AuthService


RESOURCE = 'oauth'
TEMPLATE = 'oauth_authorized.html'


class OAuthResource(AuthResource):
    internal_resource = True


class OAuthService(AuthService):

    def authenticate(self, document):
        if not document.get('email'):
            return
        return superdesk.get_resource_service('auth_users').find_one(req=None, email=document.get('email'))


def auth_user(email):
    """Authenticate user via email.

    This will create new session for user and render template with session data
    which is used by client to setup authentication.

    :param email: user email address
    """
    data = [{'email': email}]
    if not email:
        return render_template(TEMPLATE, data={'error': 404})
    try:
        superdesk.get_resource_service(RESOURCE).post(data)
        data[0]['_id'] = str(data[0]['_id'])
        data[0]['user'] = str(data[0]['user'])
        return render_template(TEMPLATE, data=data[0])
    except ValueError:
        return render_template(TEMPLATE, data={'error': 404})


def init_app(app):
    superdesk.register_resource(RESOURCE, OAuthResource, OAuthService, _app=app)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2019 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk
from .services import PlanningService, EventsService, AssignmentsService
from .resources import PlanningResource, EventsResource, AssignmentsResource


def init_app(app):
    """Initialize the `planning`, `events` and `assignments` API endpoint.

    :param app: the API application object
    :type app: `Eve`
    """
    planning_service = PlanningService(datasource='planning', backend=superdesk.get_backend())
    PlanningResource(endpoint_name='planning', app=app, service=planning_service)

    events_service = EventsService(datasource='events', backend=superdesk.get_backend())
    EventsResource(endpoint_name='events', app=app, service=events_service)

    assignments_service = AssignmentsService(datasource='assignments', backend=superdesk.get_backend())
    AssignmentsResource(endpoint_name='assignments', app=app, service=assignments_service)




import io
import eve
import bson
import unittest
from unittest.mock import Mock
from superdesk.upload import bp, upload_url
from superdesk.datalayer import SuperdeskDataLayer
from superdesk.storage import SuperdeskGridFSMediaStorage
from superdesk.utc import utcnow
from superdesk.utils import sha
from datetime import timedelta


class GridFSMediaStorageTestCase(unittest.TestCase):

    def setUp(self):
        self.app = eve.Eve(__name__, {'DOMAIN': {}})
        self.app.config['MEDIA_PREFIX'] = 'http://localhost/upload-raw'
        self.app.config['DOMAIN'] = {'upload': {}}
        self.app.config['MONGO_DBNAME'] = 'sptests'
        self.app.data = SuperdeskDataLayer(self.app)
        self.media = SuperdeskGridFSMediaStorage(self.app)
        self.app.register_blueprint(bp)
        self.app.upload_url = upload_url

    def test_url_for_media(self):
        _id = bson.ObjectId(sha('test')[:24])
        with self.app.app_context():
            url = self.media.url_for_media(_id)
        self.assertEqual('http://localhost/upload-raw/%s' % _id, url)

    def test_url_for_media_content_type(self):
        _id_str = '1' * 24
        _id = bson.ObjectId(_id_str)
        with self.app.app_context():
            url = self.media.url_for_media(_id, "image/jpeg")
        self.assertEqual('http://localhost/upload-raw/{}.jpg'.format(_id_str), url)

    def test_put_media_with_id(self):
        data = io.StringIO("test data")
        filename = 'x'

        gridfs = self._mock_gridfs()
        _id = bson.ObjectId()

        with self.app.app_context():
            self.media.put(data, filename, 'text/plain', _id=str(_id))

        kwargs = {
            'content_type': 'text/plain',
            'filename': filename,
            'metadata': None,
            '_id': _id,
        }

        gridfs.put.assert_called_once_with(data, **kwargs)

    def test_put_into_folder(self):
        data = b'test data'
        filename = 'x'
        folder = 'gridtest'

        gridfs = self._mock_gridfs()

        with self.app.app_context():
            self.media.put(data, filename=filename, content_type='text/plain', folder=folder)

        kwargs = {
            'content_type': 'text/plain',
            'filename': '{}/{}'.format(folder, filename),
            'metadata': None
        }

        gridfs.put.assert_called_once_with(data, **kwargs)

    def test_find_files(self):
        gridfs = self._mock_gridfs()
        upload_date = {'$lte': utcnow(), '$gte': utcnow() - timedelta(hours=1)}
        folder = 'gridtest'
        query_filename = {'filename': {'$regex': '^{}/'.format(folder)}}
        query_upload_date = {'uploadDate': upload_date}

        with self.app.app_context():
            self.media.find(folder=folder, upload_date=upload_date)
            gridfs.find.assert_called_once_with({'$and': [query_filename, query_upload_date]})

            self.media.find(folder=folder)
            gridfs.find.assert_called_with(query_filename)

            self.media.find(upload_date=upload_date)
            gridfs.find.assert_called_with(query_upload_date)

            self.media.find()
            gridfs.find.assert_called_with({})

    def test_custom_id(self):
        data = b'foo'
        with self.app.app_context():
            self.media.put(data, _id='foo')
            _file = self.media.get('foo')
            assert data == _file.read()

    def _mock_gridfs(self):
        gridfs = Mock()
        gridfs.put = Mock(return_value='y')
        gridfs.find = Mock(return_value=[])
        self.media._fs['MONGO'] = gridfs
        return gridfs



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013-2019 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import abc
import logging
from collections import namedtuple

logger = logging.getLogger(__name__)

Spellchecker = namedtuple("Spellchecker", ["name"])
registered_spellcheckers = {}


class SpellcheckerCapacities:
    all_capacities = {'spelling', 'grammar'}

    def __init__(self, capacities):
        if isinstance(capacities, str):
            capacities = {capacities}
        capacities = set(capacities)
        if not capacities <= self.all_capacities:
            raise ValueError("invalid capacities: {invalid}".format(
                invalid=capacities - self.all_capacities))
        self.capacities = capacities

    def serialize(self):
        return sorted(self.capacities)


class SpellcheckerRegisterer(abc.ABCMeta):

    def __call__(cls, *args, **kwargs):
        instance = super().__call__(*args, **kwargs)
        name = instance.name
        if name in registered_spellcheckers:
            # we log an error but don't raise an exception because the issue
            # may happen with tests
            logger.error('"{name}" spellchecker is already registered'.format(name=name))
            return registered_spellcheckers[name]

        instance.capacities = SpellcheckerCapacities(instance.capacities)
        if not instance.available():
            logger.warning('"{name}" spellchecker is not available'.format(name=name))
            return None
        registered_spellcheckers[name] = instance
        return instance


class SpellcheckerBase(metaclass=SpellcheckerRegisterer):
    """Base class for spellchecker.

    This class define the attribute and methods mandatory to implement.
    Spellcheckers are automatically instanciated and registered, you just need to
    inherit from this class in your module.
    "label" attribute can be used, if it is not present the name will be used as label
    """

    CHECK_TIMEOUT = (3, 5)
    SUGGEST_TIMEOUT = (3, 10)

    #: what this spellchecker can do (spelling, grammar)
    capacities = "spelling"

    #: version of the spellchecker, None if unknown
    version = None

    def __init__(self, app):
        self.config = app.config

    @property
    @abc.abstractmethod
    def name(self):
        pass

    @property
    def label(self):
        return self.name.title()

    @property
    @abc.abstractmethod
    def languages(self):
        """List of RFC-5646 tags for languages supported by this spellchecker

        Special value ['*'] means that any language can be supported.
        """
        pass

    @abc.abstractmethod
    def check(self, text, language=None):
        """Check spelling in given text"""
        pass

    def get_language(self, language):
        if language is None:
            language = self.languages[0]
        return language.split('-', 1)[0].lower()

    def suggest(self, text, language=None):
        """Get suggestions to correct given text"""
        logger.debug(u'"suggest" is not implemented')
        return {'suggestions': []}

    def available(self):
        """Return True if the spellchecker is available

        override this method is spellcheck availability depends of something (e.g. a
        server launched)
        """
        return True

    def list2suggestions(self, suggest_list):
        """Convert a list of str to expected list of objects for suggestions

        :param list suggest_list: if of suggests strings
        :return list: list of suggestions in the dict format expected by client
        """
        return [{'text': s} for s in suggest_list]

    def serialize(self):
        data = {
            'name': self.name,
            'label': self.label,
            'capacities': self.capacities.serialize(),
            'languages': self.languages,
        }
        if self.version is not None:
            data['version'] = self.version
        return data



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import functools as ft
import logging
import superdesk
from flask import current_app as app
from eve.utils import config, ParsedRequest
from copy import deepcopy
from apps.packages import PackageService
from superdesk.celery_task_utils import get_lock_id
from superdesk.utc import utcnow
from .archive import SOURCE as ARCHIVE
from superdesk.metadata.item import ITEM_STATE, CONTENT_STATE, ITEM_TYPE,\
    CONTENT_TYPE, ASSOCIATIONS, MEDIA_TYPES, PUBLISH_STATES
from superdesk.lock import lock, unlock, remove_locks
from superdesk.notification import push_notification
from superdesk import get_resource_service
from bson.objectid import ObjectId
from datetime import timedelta

logger = logging.getLogger(__name__)


def log_exeption(fn):
    @ft.wraps(fn)
    def inner(*a, **kw):
        try:
            return fn(*a, **kw)
        except Exception as e:
            logger.exception(e)
    return inner


class RemoveExpiredContent(superdesk.Command):
    """Remove expired content from Superdesk.

    It removes expired items from production, published and archived colections.

    Example:
    ::

        $ python manage.py archive:remove_expired

    """

    log_msg = ''

    def run(self):
        now = utcnow()
        self.log_msg = 'Expiry Time: {}.'.format(now)
        logger.info('{} Starting to remove expired content at.'.format(self.log_msg))
        lock_name = get_lock_id('archive', 'remove_expired')

        if not lock(lock_name, expire=610):
            logger.info('{} Remove expired content task is already running.'.format(self.log_msg))
            return

        logger.info('{} Removing expired content for expiry.'.format(self.log_msg))
        # both functions should be called, even the first one throw exception,
        # so they are wrapped with log_exeption
        self._remove_expired_publish_queue_items()
        self._remove_expired_items(now)
        unlock(lock_name)

        push_notification('content:expired')
        logger.info('{} Completed remove expired content.'.format(self.log_msg))

        remove_locks()

    @log_exeption
    def _remove_expired_publish_queue_items(self):
        expire_interval = app.config.get('PUBLISH_QUEUE_EXPIRY_MINUTES', 0)
        if expire_interval:
            expire_time = utcnow() - timedelta(minutes=expire_interval)
            logger.info('{} Removing publish queue items created before {}'.format(self.log_msg, str(expire_time)))

            get_resource_service('publish_queue').delete({'_id': {'$lte': ObjectId.from_datetime(expire_time)}})

    @log_exeption
    def _remove_expired_items(self, expiry_datetime):
        """Remove the expired items.

        :param datetime expiry_datetime: expiry datetime
        :param str log_msg: log message to be prefixed
        """
        logger.info('{} Starting to remove published expired items.'.format(self.log_msg))
        archive_service = get_resource_service(ARCHIVE)
        published_service = get_resource_service('published')
        items_to_remove = set()
        items_to_be_archived = dict()
        items_having_issues = dict()
        preserve_published_desks = {desk.get(config.ID_FIELD): 1 for desk in
                                    get_resource_service('desks').find(where={'preserve_published_content': True})}

        for expired_items in archive_service.get_expired_items(expiry_datetime):
            if len(expired_items) == 0:
                logger.info('{} No items found to expire.'.format(self.log_msg))
                return

            # delete spiked items
            self.delete_spiked_items(expired_items)

            # get killed items
            killed_items = {item.get(config.ID_FIELD): item
                            for item in expired_items
                            if item.get(ITEM_STATE) in {CONTENT_STATE.KILLED, CONTENT_STATE.RECALLED}}

            # check if killed items imported to legal
            items_having_issues.update(self.check_if_items_imported_to_legal_archive(killed_items))

            # filter out the killed items not imported to legal.
            killed_items = {item_id: item for item_id, item in killed_items.items()
                            if item_id not in items_having_issues}

            # Get the not killed and spiked items
            not_killed_items = {item.get(config.ID_FIELD): item for item in expired_items
                                if item.get(ITEM_STATE) not in {
                                    CONTENT_STATE.KILLED, CONTENT_STATE.SPIKED, CONTENT_STATE.RECALLED}}

            log_msg_format = "{{'_id': {_id}, 'unique_name': {unique_name}, 'version': {_current_version}, " \
                             "'expired_on': {expiry}}}."

            # Processing items to expire
            for item_id, item in not_killed_items.items():
                item.setdefault(config.VERSION, 1)
                item.setdefault('expiry', expiry_datetime)
                item.setdefault('unique_name', '')
                expiry_msg = log_msg_format.format(**item)
                logger.info('{} Processing expired item. {}'.format(self.log_msg, expiry_msg))

                processed_items = dict()
                if item_id not in items_to_be_archived and item_id not in items_having_issues and \
                        self._can_remove_item(item, processed_items, preserve_published_desks):
                    # item can be archived and removed from the database
                    logger.info('{} Removing item. {}'.format(self.log_msg, expiry_msg))
                    logger.info('{} Items to be removed. {}'.format(self.log_msg, processed_items))
                    issues = self.check_if_items_imported_to_legal_archive(processed_items)
                    if issues:
                        items_having_issues.update(processed_items)
                    else:
                        items_to_be_archived.update(processed_items)

            # all items to expire
            items_to_expire = deepcopy(items_to_be_archived)

            # check once again in items imported to legal
            items_having_issues.update(self.check_if_items_imported_to_legal_archive(items_to_expire))
            if items_having_issues:
                # remove items not imported to legal
                items_to_expire = {item_id: item for item_id, item in items_to_expire.items()
                                   if item_id not in items_having_issues}

                # remove items not imported to legal from archived items
                items_to_be_archived = {item_id: item for item_id, item in items_to_be_archived.items()
                                        if item_id not in items_having_issues}

                # items_to_be_archived might contain killed items
                for item_id, item in items_to_be_archived.items():
                    if item.get(ITEM_STATE) in {CONTENT_STATE.KILLED, CONTENT_STATE.RECALLED}:
                        killed_items[item_id] = item

                # remove killed items from the items_to_be_archived
                items_to_be_archived = {item_id: item for item_id, item in items_to_be_archived.items()
                                        if item.get(ITEM_STATE) not in {CONTENT_STATE.KILLED, CONTENT_STATE.RECALLED}}

            # add killed items to items to expire
            items_to_expire.update(killed_items)

            # get the filter conditions
            logger.info('{} filter conditions.'.format(self.log_msg))
            req = ParsedRequest()
            filter_conditions = list(get_resource_service('content_filters').get(req=req,
                                                                                 lookup={'is_archived_filter': True}))

            # move to archived collection
            logger.info('{} Archiving items.'.format(self.log_msg))
            for item_id, item in items_to_be_archived.items():
                self._move_to_archived(item, filter_conditions)

            for item_id, item in killed_items.items():
                # delete from the published collection and queue
                msg = log_msg_format.format(**item)
                try:
                    published_service.delete_by_article_id(item_id)
                    logger.info('{} Deleting killed item from published. {}'.format(self.log_msg, msg))
                    items_to_remove.add(item_id)
                except Exception:
                    logger.exception('{} Failed to delete killed item from published. {}'.format(self.log_msg, msg))

            if items_to_remove:
                logger.info('{} Deleting articles.: {}'.format(self.log_msg, items_to_remove))
                archive_service.delete_by_article_ids(list(items_to_remove))

            for item_id, item in items_having_issues.items():
                msg = log_msg_format.format(**item)
                try:
                    archive_service.system_update(item.get(config.ID_FIELD), {'expiry_status': 'invalid'}, item)
                    logger.info('{} Setting item expiry status. {}'.format(self.log_msg, msg))
                except Exception:
                    logger.exception('{} Failed to set expiry status for item. {}'.format(self.log_msg, msg))

            logger.info('{} Deleting killed from archive.'.format(self.log_msg))

    def _can_remove_item(self, item, processed_item=None, preserve_published_desks=None):
        """Recursively checks if the item can be removed.

        :param dict item: item to be remove
        :param set processed_item: processed items
        :return: True if item can be removed, False otherwise.
        """

        if processed_item is None:
            processed_item = dict()

        item_refs = []
        package_service = PackageService()
        archive_service = get_resource_service(ARCHIVE)

        if item.get(ITEM_TYPE) == CONTENT_TYPE.COMPOSITE:
            # Get the item references for is package
            item_refs = package_service.get_residrefs(item)

        if item.get(ITEM_TYPE) in [CONTENT_TYPE.TEXT, CONTENT_TYPE.PREFORMATTED]:
            broadcast_items = get_resource_service('archive_broadcast').get_broadcast_items_from_master_story(item)
            # If master story expires then check if broadcast item is included in a package.
            # If included in a package then check the package expiry.
            item_refs.extend([broadcast_item.get(config.ID_FIELD) for broadcast_item in broadcast_items])

            if item.get('rewrite_of'):
                item_refs.append(item.get('rewrite_of'))

            if item.get('rewritten_by'):
                item_refs.append(item.get('rewritten_by'))

        # get the list of associated item ids
        if item.get(ITEM_TYPE) in MEDIA_TYPES:
            item_refs.extend(self._get_associated_items(item))

        # get item reference where this referred
        item_refs.extend(package_service.get_linked_in_package_ids(item))

        # check item refs in the ids to remove set
        is_expired = item.get('expiry') and item.get('expiry') < utcnow()

        # if the item is published or corrected and desk has preserve_published_content as true
        if preserve_published_desks and \
                item.get(ITEM_STATE) in {CONTENT_STATE.PUBLISHED, CONTENT_STATE.CORRECTED} and \
                item.get('task').get('desk') in preserve_published_desks:
            is_expired = False

        # If the item is associated with a planning assignment and not published then preserve it
        if item.get('assignment_id') and item.get(ITEM_STATE) not in PUBLISH_STATES:
            is_expired = False

        if is_expired:
            # now check recursively for all references
            if item.get(config.ID_FIELD) in processed_item:
                return is_expired

            processed_item[item.get(config.ID_FIELD)] = item
            if item_refs:
                archive_items = archive_service.get_from_mongo(req=None, lookup={'_id': {'$in': item_refs}})
                for archive_item in archive_items:
                    is_expired = self._can_remove_item(archive_item, processed_item, preserve_published_desks)
                    if not is_expired:
                        break

        # If this item is not expired then it is potentially keeping it's parent alive.
        if not is_expired:
            logger.info('{} Item ID: [{}] has not expired'.format(self.log_msg, item.get(config.ID_FIELD)))
        return is_expired

    def _get_associated_media_id(self, item):
        """Get the associated media id
        :param dict item:
        :return list: list of associated item ids
        """
        ids = []
        for key, associated_item in (item.get(ASSOCIATIONS) or {}).items():
            if associated_item:
                ids.append(associated_item.get(config.ID_FIELD))
        return ids

    def _get_associated_items(self, item):
        """Get the list of item ids where the media item is associated.
        :param dict item: media item
        :return list: list of associated item ids
        """
        if item.get(ITEM_TYPE) not in MEDIA_TYPES:
            return []

        associated_items = list(get_resource_service('media_references').get(req=None,
                                                                             lookup={
                                                                                 'associated_id':
                                                                                     item.get(config.ID_FIELD)
                                                                             }))

        ids = set()
        for assoc in associated_items:
            ids.add(assoc['item_id'])

        # extra query just to ensure that item is not deleted
        # get the associated items from the archive collection
        archive_docs = list(get_resource_service(ARCHIVE).get_from_mongo(req=None,
                                                                         lookup={'_id': {'$in': list(ids)}}))
        return [doc.get(config.ID_FIELD) for doc in archive_docs]

    def _move_to_archived(self, item, filter_conditions):
        """Moves all the published version of an article to archived.

        Deletes all published version of an article in the published collection

        :param str item_id: item_id of the document
        :param list filter_conditions: list of filter conditions
        """
        published_service = get_resource_service('published')
        archived_service = get_resource_service('archived')
        archive_service = get_resource_service('archive')
        item_id = item.get(config.ID_FIELD)
        moved_to_archived = self._conforms_to_archived_filter(item, filter_conditions)
        published_items = list(published_service.get_from_mongo(req=None, lookup={'item_id': item_id}))

        try:
            if published_items:
                # moved to archive
                logger.info('{} Found {} published items for item: {}'.format(self.log_msg,
                                                                              len(published_items), item_id))
                if moved_to_archived:
                    archived_service.post(published_items)
                    logger.info('{} Moved item to text archive for item {}.'.format(self.log_msg, item_id))
                else:
                    logger.info('{} Not Moving item to text archive for item {}.'.format(self.log_msg, item_id))

                logger.info('{} Archived published item: {}'.format(self.log_msg, item_id))
                published_service.delete_by_article_id(item_id)
                logger.info('{} Deleted published item. {}'.format(self.log_msg, item_id))

            archive_service.delete_by_article_ids([item_id])
            logger.info('{} Deleted archive item. {}'.format(self.log_msg, item_id))
        except Exception:
            failed_items = [item.get(config.ID_FIELD) for item in published_items]
            logger.exception('{} Failed to move to archived. {}'.format(self.log_msg, failed_items))

    def _conforms_to_archived_filter(self, item, filter_conditions):
        """Check if the item can be moved the archived collection or not.

        :param dict item: item to be moved
        :param list filter_conditions: list of filter conditions
        :return bool: True to archive the item else False
        """
        if not filter_conditions:
            logger.info('{} No filter conditions specified for Archiving item {}.'.format(self.log_msg,
                                                                                          item.get(config.ID_FIELD)))
            return True

        filter_service = get_resource_service('content_filters')
        for fc in filter_conditions:
            if filter_service.does_match(fc, item):
                logger.info('{} Filter conditions {} matched for item {}.'.format(self.log_msg, fc,
                                                                                  item.get(config.ID_FIELD)))
                return False

        logger.info('{} No filter conditions matched Archiving item {}.'.format(self.log_msg,
                                                                                item.get(config.ID_FIELD)))
        return True

    def delete_spiked_items(self, items):
        """Delete spiked items

        :param list items:
        """
        try:
            logger.info('{} deleting spiked items.'.format(self.log_msg))
            spiked_ids = [item.get(config.ID_FIELD) for item in items
                          if item.get(ITEM_STATE) == CONTENT_STATE.SPIKED and not self._get_associated_items(item)]
            if spiked_ids:
                logger.warning('{} deleting spiked items: {}.'.format(self.log_msg, spiked_ids))
                get_resource_service('archive').delete_by_article_ids(spiked_ids)
            logger.info('{} deleted spiked items. Count: {}.'.format(self.log_msg, len(spiked_ids)))
        except Exception:
            logger.exception('{} Failed to delete spiked items.'.format(self.log_msg))

    def check_if_items_imported_to_legal_archive(self, items_to_expire):
        """Checks if all items are moved to legal or not.

        :param dict items_to_expire:
        :return dict: dict of items having issues.
        """
        logger.info('{} checking for items in legal archive. Items: {}'.format(self.log_msg,
                                                                               items_to_expire.keys()))

        items_not_moved_to_legal = get_resource_service('published').\
            get_published_items_by_moved_to_legal(list(items_to_expire.keys()), False)

        items_not_moved = dict()
        if len(items_not_moved_to_legal) > 0:
            publish_items = set([item.get('item_id') for item in items_not_moved_to_legal])
            items_not_moved = {item_id: items_to_expire[item_id] for item_id in publish_items}
            logger.warning('{} Items are not moved to legal archive {}.'.format(self.log_msg, publish_items))

        # get all the
        lookup = {
            '$and': [
                {'item_id': {'$in': list(items_to_expire.keys())}},
                {'moved_to_legal': False}
            ]
        }

        items_not_moved_to_legal = list(get_resource_service('publish_queue').get(req=None, lookup=lookup))

        if len(items_not_moved_to_legal) > 0:
            publish_queue_items = set([item.get('item_id') for item in items_not_moved_to_legal])
            items_not_moved.update({item_id: items_to_expire[item_id] for item_id in publish_queue_items})
            logger.warning('{} Items are not moved to legal publish queue {}.'.format(self.log_msg,
                                                                                      publish_queue_items))

        return items_not_moved


superdesk.command('archive:remove_expired', RemoveExpiredContent())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
from flask import current_app as app
from superdesk.errors import SuperdeskApiError
import superdesk
from .ldap import ADAuth, add_default_values, get_user_query
from flask_babel import _

logger = logging.getLogger(__name__)


class ImportUserProfileFromADCommand(superdesk.Command):
    """Responsible for importing a user profile from Active Directory (AD) to Mongo.

    This command runs on assumption that the user executing this command and
    the user whose profile need to be imported need not to be the same. Uses ad_username and ad_password to bind to AD
    and then searches for a user identified by username_to_import and if found imports into Mongo.

    Example:
    ::

        $ python manage.py users:copyfromad --ad_username=ad_uname --ad_password=123 --username_to_import=admin

    """

    option_list = (
        superdesk.Option('--ad_username', '-adu', dest='ad_username', required=True),
        superdesk.Option('--ad_password', '-adp', dest='ad_password', required=True),
        superdesk.Option('--username_to_import', '-u', dest='username', required=True),
        superdesk.Option('--admin', '-a', dest='admin', required=False),
    )

    def run(self, ad_username, ad_password, username, admin='false'):
        """Imports or Updates a User Profile from AD to Mongo.

        :param ad_username: Active Directory Username
        :param ad_password: Password of Active Directory Username
        :param username: Username as in Active Directory whose profile needs to be imported to Superdesk.
        :return: User Profile.
        """

        # force type conversion to boolean
        user_type = 'administrator' if admin is not None and admin.lower() == 'true' else 'user'

        # Authenticate and fetch profile from AD
        settings = app.settings
        ad_auth = ADAuth(settings['LDAP_SERVER'], settings['LDAP_SERVER_PORT'], settings['LDAP_BASE_FILTER'],
                         settings['LDAP_USER_FILTER'], settings['LDAP_USER_ATTRIBUTES'], settings['LDAP_FQDN'])

        user_data = ad_auth.authenticate_and_fetch_profile(ad_username, ad_password, username)

        if len(user_data) == 0:
            raise SuperdeskApiError.notFoundError('Username not found')

        # Check if User Profile already exists in Mongo
        user = superdesk.get_resource_service('users').find_one(req=None, **get_user_query(username))

        if user:
            superdesk.get_resource_service('users').patch(user.get('_id'), user_data)
        else:
            add_default_values(user_data, username, user_type=user_type)
            superdesk.get_resource_service('users').post([user_data])

        return user_data


superdesk.command('users:copyfromad', ImportUserProfileFromADCommand())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


from .ninjs_formatter import NINJSFormatter
import superdesk


class NewsroomNinjsFormatter(NINJSFormatter):
    def __init__(self):
        self.format_type = 'newsroom ninjs'
        self.can_preview = False
        self.can_export = False
        self.internal_renditions = ['original', 'viewImage', 'baseImage']

    def _format_products(self, article):
        """
        Return a list of API product id's that the article matches.

        :param article:
        :return:
        """
        result = superdesk.get_resource_service('product_tests').test_products(article, lookup=None)
        return [{'code': p['product_id'], 'name': p.get('name')} for p in result if p.get('matched', False)]

    def _transform_to_ninjs(self, article, subscriber, recursive=True):
        ninjs = super()._transform_to_ninjs(article, subscriber, recursive)

        if article.get('ingest_id') and article.get('auto_publish'):
            ninjs['guid'] = article.get('ingest_id')
            ninjs['version'] = article.get('ingest_version')

        ninjs['products'] = self._format_products(article)

        if article.get('assignment_id'):
            assignment = superdesk.get_resource_service('assignments').find_one(req=None, _id=article['assignment_id'])
            if assignment is not None:
                if assignment.get('coverage_item'):
                    ninjs.setdefault('coverage_id', assignment['coverage_item'])
                if assignment.get('planning_item'):
                    ninjs.setdefault('planning_id', assignment['planning_item'])

        return ninjs




import flask
import unittest
import unittest.mock
from . import push_content_notification


class PushContentNotificationTestCase(unittest.TestCase):

    def setUp(self):
        self.app = flask.Flask(__name__)
        self.ctx = self.app.app_context()
        self.ctx.push()

    def tearDown(self):
        self.ctx.pop()

    @unittest.mock.patch('apps.content.push_notification')
    def test_push_content_notification(self, push_notification):
        foo1 = {'_id': 'foo', 'task': {'desk': 'sports', 'stage': 'inbox'}}
        foo2 = {'_id': 'foo', 'task': {'desk': 'news', 'stage': 'todo'}}
        foo3 = {'_id': 'foo'}

        push_content_notification([foo1, foo2, foo3])
        push_notification.assert_called_once_with(
            'content:update',
            user='',
            items={'foo': 1},
            desks={'sports': 1, 'news': 1},
            stages={'inbox': 1, 'todo': 1}
        )



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from .common import BasePublishResource, ITEM_TAKEDOWN, ITEM_KILL
from .kill import KillPublishService
from superdesk.metadata.item import CONTENT_STATE
import logging

logger = logging.getLogger(__name__)


class TakeDownPublishResource(BasePublishResource):
    def __init__(self, endpoint_name, app, service):
        super().__init__(endpoint_name, app, service, ITEM_TAKEDOWN)


class TakeDownPublishService(KillPublishService):
    publish_type = ITEM_KILL
    published_state = CONTENT_STATE.RECALLED
    item_operation = ITEM_TAKEDOWN

    def __init__(self, datasource=None, backend=None):
        super().__init__(datasource=datasource, backend=backend)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk
from superdesk.resource import Resource
from superdesk.services import BaseService
from apps.comments import CommentsService, CommentsResource, comments_schema
from superdesk.errors import SuperdeskApiError


comments_schema = dict(comments_schema)
comments_schema.update({'item': Resource.rel('archive', True, True, type='string')})


class ItemCommentsResource(CommentsResource):
    schema = comments_schema
    resource_methods = ['GET', 'POST', 'DELETE']
    datasource = {'default_sort': [('_created', -1)]}
    privileges = {'POST': 'archive', 'DELETE': 'archive'}


class ItemCommentsService(CommentsService):
    notification_key = 'item:comment'


class ItemCommentsSubResource(Resource):
    url = 'archive/<path:item>/comments'
    schema = comments_schema
    datasource = {'source': 'item_comments'}
    resource_methods = ['GET']


class ItemCommentsSubService(BaseService):

    def check_item_valid(self, item_id):
        item = superdesk.get_resource_service('archive').find_one(req=None, _id=item_id)
        if not item:
            msg = 'Invalid content item ID provided: %s' % item_id
            raise SuperdeskApiError.notFoundError(msg)

    def get(self, req, lookup):
        self.check_item_valid(lookup.get('item'))
        return super().get(req, lookup)


def init_app(app):
    endpoint_name = 'item_comments'
    service = ItemCommentsService(endpoint_name, backend=superdesk.get_backend())
    ItemCommentsResource(endpoint_name, app=app, service=service)

    endpoint_name = 'content_item_comments'
    service = ItemCommentsSubService(endpoint_name, backend=superdesk.get_backend())
    ItemCommentsSubResource(endpoint_name, app=app, service=service)
